@@ -35,7 +35,7 @@ Torus Knot Software Ltd.
 #include "OgreIteratorWrappers.h"
 #include "OgreAnimable.h"
 #include "OgreAnimationTrack.h"
-
+#include "OgreAnimationState.h"
 
 namespace Ogre {
 
@@ -49,7 +49,7 @@ namespace Ogre {
         You should not create these animations directly. They will be created via a parent
         object which owns the animation, e.g. Skeleton.
     */
-    class _OgreExport Animation
+	class _OgreExport Animation : public AnimationAlloc
     {
 
     public:
@@ -87,6 +87,12 @@ namespace Ogre {
         /** Gets the total length of the animation. */
         Real getLength(void) const;
 
+		/** Sets the length of the animation. 
+		@note Changing the length of an animation may invalidate existing AnimationState
+		instances which will need to be recreated. 
+		*/
+		void setLength(Real len);
+
         /** Creates a NodeAnimationTrack for animating a Node.
         @param handle Handle to give the track, used for accessing the track later. 
             Must be unique within this Animation.
@@ -183,7 +189,7 @@ namespace Ogre {
 
         /** Applies an animation given a specific time point and weight.
         @remarks
-            Where you have associated animation tracks with objects, you can eaily apply
+            Where you have associated animation tracks with objects, you can easily apply
             an animation to those objects by calling this method.
         @param timePos The time position in the animation to apply.
         @param weight The influence to give to this track, 1.0 for full influence, less to blend with
@@ -195,7 +201,7 @@ namespace Ogre {
 
         /** Applies all node tracks given a specific time point and weight to a given skeleton.
         @remarks
-        Where you have associated animation tracks with Node objects, you can eaily apply
+        Where you have associated animation tracks with Node objects, you can easily apply
         an animation to those nodes by calling this method.
         @param timePos The time position in the animation to apply.
         @param weight The influence to give to this track, 1.0 for full influence, less to blend with
@@ -205,6 +211,21 @@ namespace Ogre {
         */
         void apply(Skeleton* skeleton, Real timePos, Real weight = 1.0, Real scale = 1.0f);
 
+        /** Applies all node tracks given a specific time point and weight to a given skeleton.
+        @remarks
+        Where you have associated animation tracks with Node objects, you can easily apply
+        an animation to those nodes by calling this method.
+        @param timePos The time position in the animation to apply.
+        @param weight The influence to give to this track, 1.0 for full influence, less to blend with
+        other animations.
+        @param blendMask The influence array defining additional per bone weights. These will
+        be modulated with the weight factor.
+        @param scale The scale to apply to translations and scalings, useful for 
+        adapting an animation to a different size target.
+        */
+        void apply(Skeleton* skeleton, Real timePos, float weight,
+          const AnimationState::BoneBlendMask* blendMask, Real scale);
+
 		/** Applies all vertex tracks given a specific time point and weight to a given entity.
 		@remarks
 		@param entity The Entity to which this animation should be applied
