@@ -48,19 +48,19 @@ namespace Ogre {
             parameters like type, position, attenuation (how light intensity fades with
             distance), colour etc.
         @par
-            The defaults when a light is created is pure white diffues light, with no
+            The defaults when a light is created is pure white diffuse light, with no
             attenuation (does not decrease with distance) and a range of 1000 world units.
         @par
             Lights are created by using the SceneManager::createLight method. They can subsequently be
             added to a SceneNode if required to allow them to move relative to a node in the scene. A light attached
-            to a SceneNode is assumed to havea base position of (0,0,0) and a direction of (0,0,1) before modification
+            to a SceneNode is assumed to have a base position of (0,0,0) and a direction of (0,0,1) before modification
             by the SceneNode's own orientation. If not attached to a SceneNode,
             the light's position and direction is as set using setPosition and setDirection.
         @par
             Remember also that dynamic lights rely on modifying the colour of vertices based on the position of
             the light compared to an object's vertex normals. Dynamic lighting will only look good if the
-            object being lit has a fair level of tesselation and the normals are properly set. This is particularly
-            true for the spotlight which will only look right on highly tesselated models. In the future OGRE may be
+            object being lit has a fair level of tessellation and the normals are properly set. This is particularly
+            true for the spotlight which will only look right on highly tessellated models. In the future OGRE may be
             extended for certain scene types so an alternative to the standard dynamic lighting may be used, such
             as dynamic lightmaps.
     */
@@ -69,16 +69,18 @@ namespace Ogre {
     public:
         /// Temp tag used for sorting
         Real tempSquareDist;
+		/// internal method for calculating current squared distance from some world position
+		void _calcTempSquareDist(const Vector3& worldPos);
 
         /// Defines the type of light
         enum LightTypes
         {
             /// Point light sources give off light equally in all directions, so require only position not direction
-            LT_POINT,
+            LT_POINT = 0,
             /// Directional lights simulate parallel light beams from a distant source, hence have direction but no position
-            LT_DIRECTIONAL,
+            LT_DIRECTIONAL = 1,
             /// Spotlights simulate a cone of light from a source so require position and direction, plus extra values for falloff
-            LT_SPOTLIGHT
+            LT_SPOTLIGHT = 2
         };
 
         /** Default constructor (for Python mainly).
@@ -107,7 +109,7 @@ namespace Ogre {
                 light an object reflects. This value denotes the amount and colour of this type of light the light
                 exudes into the scene. The actual appearance of objects is a combination of the two.
             @par
-                Diffuse light simulates the typical light emenating from light sources and affects the base colour
+                Diffuse light simulates the typical light emanating from light sources and affects the base colour
                 of objects together with ambient light.
         */
         void setDiffuseColour(Real red, Real green, Real blue);
@@ -118,7 +120,7 @@ namespace Ogre {
                 light an object reflects. This value denotes the amount and colour of this type of light the light
                 exudes into the scene. The actual appearance of objects is a combination of the two.
             @par
-                Diffuse light simulates the typical light emenating from light sources and affects the base colour
+                Diffuse light simulates the typical light emanating from light sources and affects the base colour
                 of objects together with ambient light.
         */
         void setDiffuseColour(const ColourValue& colour);
@@ -153,12 +155,12 @@ namespace Ogre {
         */
         const ColourValue& getSpecularColour(void) const;
 
-        /** Sets the attenuation parameters of the light source ie how it diminishes with distance.
+        /** Sets the attenuation parameters of the light source i.e. how it diminishes with distance.
             @remarks
                 Lights normally get fainter the further they are away. Also, each light is given a maximum range
                 beyond which it cannot affect any objects.
             @par
-                Light attentuation is not applicable to directional lights since they have an infinite range and
+                Light attenuation is not applicable to directional lights since they have an infinite range and
                 constant intensity.
             @par
                 This follows a standard attenuation approach - see any good 3D text for the details of what they mean
@@ -245,11 +247,6 @@ namespace Ogre {
                 falloff The rate of falloff between the inner and outer cones. 1.0 means a linear falloff, less means slower falloff, higher means faster falloff.
         */
         void setSpotlightRange(const Radian& innerAngle, const Radian& outerAngle, Real falloff = 1.0);
-#ifndef OGRE_FORCE_ANGLE_TYPES
-        inline void setSpotlightRange(Real innerAngle, Real outerAngle, Real falloff = 1.0) {
-            setSpotlightRange ( Angle(innerAngle), Angle(outerAngle), falloff );
-        }
-#endif//OGRE_FORCE_ANGLE_TYPES
 
         /** Returns the angle covered by the spotlights inner cone.
         */
@@ -304,8 +301,10 @@ namespace Ogre {
         /** Overridden from MovableObject */
         const String& getMovableType(void) const;
 
-        /** Retrieves the position of the light including any transform from nodes it is attached to. */
-        const Vector3& getDerivedPosition(void) const;
+        /** Retrieves the position of the light including any transform from nodes it is attached to. 
+		@param cameraRelativeIfSet If set to true, returns data in camera-relative units if that's been set up (render use)
+		*/
+        const Vector3& getDerivedPosition(bool cameraRelativeIfSet = false) const;
 
         /** Retrieves the direction of the light including any transform from nodes it is attached to. */
         const Vector3& getDerivedDirection(void) const;
@@ -327,8 +326,9 @@ namespace Ogre {
 			example the vector can represent both position lights (w=1.0f)
 			and directional lights (w=0.0f) and be used in the same 
 			calculations.
+		@param cameraRelativeIfSet If set to true, returns data in camera-relative units if that's been set up (render use)
 		*/
-		Vector4 getAs4DVector(void) const;
+		Vector4 getAs4DVector(bool cameraRelativeIfSet = false) const;
 
         /** Internal method for calculating the 'near clip volume', which is
             the volume formed between the near clip rectangle of the 
@@ -371,6 +371,50 @@ namespace Ogre {
 		/** return a pointer to the custom shadow camera setup (null means use SceneManager global version). */
 		const ShadowCameraSetupPtr& getCustomShadowCameraSetup(void) const;
 
+		/// @copydoc MovableObject::visitRenderables
+		void visitRenderables(Renderable::Visitor* visitor, 
+			bool debugRenderables = false);
+
+		/** Gets the index at which this light is in the current render. 
+		@remarks
+			Lights will be present in the in a list for every renderable,
+			detected and sorted appropriately, and sometimes it's useful to know 
+			what position in that list a given light occupies. This can vary 
+			from frame to frame (and object to object) so you should not use this
+			value unless you're sure the context is correct.
+		*/
+		size_t _getIndexInFrame() const { return mIndexInFrame; }
+		void _notifyIndexInFrame(size_t i) { mIndexInFrame = i; }
+		
+        /** Sets the maximum distance away from the camera that shadows
+        by this light will be visible.
+        @remarks
+        Shadow techniques can be expensive, therefore it is a good idea
+        to limit them to being rendered close to the camera if possible,
+        and to skip the expense of rendering shadows for distance objects.
+        This method allows you to set the distance at which shadows will no
+        longer be rendered.
+        @note
+        Each shadow technique can interpret this subtely differently.
+        For example, one technique may use this to eliminate casters,
+        another might use it to attenuate the shadows themselves.
+        You should tweak this value to suit your chosen shadow technique
+        and scene setup.
+        */
+        void setShadowFarDistance(Real distance);
+        /** Tells the light to use the shadow far distance of the SceneManager
+        */
+        void resetShadowFarDistance(void);
+        /** Gets the maximum distance away from the camera that shadows
+        by this light will be visible.
+        */
+        Real getShadowFarDistance(void) const;
+        Real getShadowFarDistanceSquared(void) const;
+
+		/// Set the camera which this light should be relative to, for camera-relative rendering
+		void _setCameraRelative(Camera* cam);
+
+
     protected:
         /// internal method for synchronising with parent node (if any)
         virtual void update(void) const;
@@ -395,10 +439,18 @@ namespace Ogre {
         Real mAttenuationLinear;
         Real mAttenuationQuad;
 		Real mPowerScale;
+		size_t mIndexInFrame;
+		bool mOwnShadowFarDist;
+		Real mShadowFarDist;
+		Real mShadowFarDistSquared;
 
 
         mutable Vector3 mDerivedPosition;
         mutable Vector3 mDerivedDirection;
+		// Slightly hacky but unless we separate observed light render state from main Light...
+		mutable Vector3 mDerivedCamRelativePosition;
+		mutable bool mDerivedCamRelativeDirty;
+		Camera* mCameraToBeRelativeTo;
 
         /// Shared class-level name for Movable type
         static String msMovableType;
