@@ -38,7 +38,21 @@ Torus Knot Software Ltd.
 #include "OgreTextureUnitState.h"
 
 namespace Ogre {
-    /** Class defining a single pass of a Technique (of a Material), ie
+
+	/// Categorisation of passes for the purpose of additive lighting
+	enum IlluminationStage
+	{
+		/// Part of the rendering which occurs without any kind of direct lighting
+		IS_AMBIENT,
+		/// Part of the rendering which occurs per light
+		IS_PER_LIGHT,
+		/// Post-lighting rendering
+		IS_DECAL, 
+		/// Not determined
+		IS_UNKNOWN
+	};
+
+    /** Class defining a single pass of a Technique (of a Material), i.e.
         a single rendering call.
     @remarks
         Rendering can be repeated with many passes for more complex effects.
@@ -57,7 +71,7 @@ namespace Ogre {
         There are pros and cons to both, just remember that if you use a programmable
         pass to create some great effects, allow more time for definition and testing.
     */
-    class _OgreExport Pass
+	class _OgreExport Pass : public PassAlloc
     {
 	public:
 		/** Definition of a functor for calculating the hashcode of a Pass.
@@ -81,6 +95,7 @@ namespace Ogre {
         unsigned short mIndex; // pass index
         String mName; // optional name for the pass
         uint32 mHash; // pass hash
+		bool mHashDirtyQueued; // needs to be dirtied when next loaded
         //-------------------------------------------------------------------------
         // Colour properties, only applicable in fixed-function passes
         ColourValue mAmbient;
@@ -95,6 +110,11 @@ namespace Ogre {
         // Blending factors
         SceneBlendFactor mSourceBlendFactor;
         SceneBlendFactor mDestBlendFactor;
+		SceneBlendFactor mSourceBlendFactorAlpha;
+		SceneBlendFactor mDestBlendFactorAlpha;
+
+		// Used to determine if separate alpha blending should be used for color and alpha channels
+		bool mSeparateBlend;
         //-------------------------------------------------------------------------
 
         //-------------------------------------------------------------------------
@@ -104,6 +124,7 @@ namespace Ogre {
         CompareFunction mDepthFunc;
         float mDepthBiasConstant;
 		float mDepthBiasSlopeScale;
+		float mDepthBiasPerIteration;
 
         // Colour buffer settings
         bool mColourWrite;
@@ -111,6 +132,10 @@ namespace Ogre {
 		// Alpha reject settings
 		CompareFunction mAlphaRejectFunc;
 		unsigned char mAlphaRejectVal;
+		bool mAlphaToCoverageEnabled;
+
+		// Transparent depth sorting
+		bool mTransparentSorting;
         //-------------------------------------------------------------------------
 
         //-------------------------------------------------------------------------
@@ -137,7 +162,9 @@ namespace Ogre {
         ShadeOptions mShadeOptions;
 		/// Polygon mode
 		PolygonMode mPolygonMode;
-
+		/// Normalisation
+		bool mNormaliseNormals;
+		bool mPolygonModeOverrideable;
         //-------------------------------------------------------------------------
         // Fog
         bool mFogOverride;
@@ -162,6 +189,8 @@ namespace Ogre {
 		GpuProgramUsage *mFragmentProgramUsage;
 		// Fragment program details
 		GpuProgramUsage *mShadowReceiverFragmentProgramUsage;
+		// Geometry program details
+		GpuProgramUsage *mGeometryProgramUsage;
         // Is this pass queued for deletion?
         bool mQueuedForDeletion;
         // number of pass iterations to perform
@@ -178,6 +207,15 @@ namespace Ogre {
 		typedef std::vector<unsigned short> ContentTypeLookup;
 		mutable ContentTypeLookup mShadowContentTypeLookup;
 		mutable bool mContentTypeLookupBuilt;
+		/// Scissoring for the light?
+		bool mLightScissoring;
+		/// User clip planes for light?
+		bool mLightClipPlanes;
+		/// Illumination stage?
+		IlluminationStage mIlluminationStage;
+
+		// Used to get scene blending flags from a blending type
+		void _getBlendFlags(SceneBlendType type, SceneBlendFactor& source, SceneBlendFactor& dest);
 
 	public:
 		typedef std::set<Pass*> PassSet;
@@ -189,22 +227,26 @@ namespace Ogre {
 		/// The Pass hash functor
 		static HashFunc* msHashFunc;
     public:
-		OGRE_STATIC_MUTEX(msDirtyHashListMutex);
-		OGRE_STATIC_MUTEX(msPassGraveyardMutex);
+		OGRE_STATIC_MUTEX(msDirtyHashListMutex)
+		OGRE_STATIC_MUTEX(msPassGraveyardMutex)
+		OGRE_MUTEX(mTexUnitChangeMutex)
+		OGRE_MUTEX(mGpuProgramChangeMutex)
         /// Default constructor
 		Pass(Technique* parent, unsigned short index);
         /// Copy constructor
         Pass(Technique* parent, unsigned short index, const Pass& oth );
         /// Operator = overload
         Pass& operator=(const Pass& oth);
-        ~Pass();
+        virtual ~Pass();
 
-        /// Returns true if this pass is programmable ie includes either a vertex or fragment program.
-        bool isProgrammable(void) const { return mVertexProgramUsage || mFragmentProgramUsage; }
+        /// Returns true if this pass is programmable i.e. includes either a vertex or fragment program.
+        bool isProgrammable(void) const { return mVertexProgramUsage || mFragmentProgramUsage || mGeometryProgramUsage; }
         /// Returns true if this pass uses a programmable vertex pipeline
         bool hasVertexProgram(void) const { return mVertexProgramUsage != NULL; }
         /// Returns true if this pass uses a programmable fragment pipeline
         bool hasFragmentProgram(void) const { return mFragmentProgramUsage != NULL; }
+        /// Returns true if this pass uses a programmable geometry pipeline
+        bool hasGeometryProgram(void) const { return mGeometryProgramUsage != NULL; }
         /// Returns true if this pass uses a shadow caster vertex program
         bool hasShadowCasterVertexProgram(void) const { return mShadowCasterVertexProgramUsage != NULL; }
         /// Returns true if this pass uses a shadow receiver vertex program
@@ -217,8 +259,8 @@ namespace Ogre {
         unsigned short getIndex(void) const { return mIndex; }
         /* Set the name of the pass
         @remarks
-        The name of the pass is optional.  Its usefull in material scripts where a material could inherit
-        from another material and only want to modify a particalar pass.
+        The name of the pass is optional.  Its useful in material scripts where a material could inherit
+        from another material and only want to modify a particular pass.
         */
         void setName(const String& name);
         /// get the name of the pass
@@ -385,7 +427,7 @@ namespace Ogre {
 			The resulting size is clamped to the minimum and maximum point
 			size.
 		@param enabled Whether point attenuation is enabled
-		@param constant, linear, quadratic Parameters to the attentuation
+		@param constant, linear, quadratic Parameters to the attenuation
 			function defined above
 		*/
 		void setPointAttenuation(bool enabled,
@@ -526,9 +568,29 @@ namespace Ogre {
         */
         void setSceneBlending( const SceneBlendType sbt );
 
+       /** Sets the kind of blending this pass has with the existing contents of the scene, separately for color and alpha channels
+        @remarks
+        Whereas the texture blending operations seen in the TextureUnitState class are concerned with
+        blending between texture layers, this blending is about combining the output of the Pass
+        as a whole with the existing contents of the rendering target. This blending therefore allows
+        object transparency and other special effects. If all passes in a technique have a scene
+        blend, then the whole technique is considered to be transparent.
+        @par
+        This method allows you to select one of a number of predefined blending types. If you require more
+        control than this, use the alternative version of this method which allows you to specify source and
+        destination blend factors.
+        @note
+        This method is applicable for both the fixed-function and programmable pipelines.
+        @param
+        sbt One of the predefined SceneBlendType blending types for the color channel
+        @param
+        sbta One of the predefined SceneBlendType blending types for the alpha channel
+        */
+        void setSeparateSceneBlending( const SceneBlendType sbt, const SceneBlendType sbta );
+
         /** Allows very fine control of blending this Pass with the existing contents of the scene.
         @remarks
-        Wheras the texture blending operations seen in the TextureUnitState class are concerned with
+        Whereas the texture blending operations seen in the TextureUnitState class are concerned with
         blending between texture layers, this blending is about combining the output of the material
         as a whole with the existing contents of the rendering target. This blending therefore allows
         object transparency and other special effects.
@@ -550,6 +612,37 @@ namespace Ogre {
         */
         void setSceneBlending( const SceneBlendFactor sourceFactor, const SceneBlendFactor destFactor);
 
+        /** Allows very fine control of blending this Pass with the existing contents of the scene.
+        @remarks
+        Wheras the texture blending operations seen in the TextureUnitState class are concerned with
+        blending between texture layers, this blending is about combining the output of the material
+        as a whole with the existing contents of the rendering target. This blending therefore allows
+        object transparency and other special effects.
+        @par
+        This version of the method allows complete control over the blending operation, by specifying the
+        source and destination blending factors. The result of the blending operation is:
+        <span align="center">
+        final = (texture * sourceFactor) + (pixel * destFactor)
+        </span>
+        @par
+        Each of the factors is specified as one of a number of options, as specified in the SceneBlendFactor
+        enumerated type.
+        @param
+        sourceFactor The source factor in the above calculation, i.e. multiplied by the texture colour components.
+        @param
+        destFactor The destination factor in the above calculation, i.e. multiplied by the pixel colour components.
+        @param
+        sourceFactorAlpha The alpha source factor in the above calculation, i.e. multiplied by the texture alpha component.
+        @param
+        destFactorAlpha The alpha destination factor in the above calculation, i.e. multiplied by the pixel alpha component.
+		@note
+        This method is applicable for both the fixed-function and programmable pipelines.
+        */
+		void setSeparateSceneBlending( const SceneBlendFactor sourceFactor, const SceneBlendFactor destFactor, const SceneBlendFactor sourceFactorAlpha, const SceneBlendFactor destFactorAlpha );
+
+		/** Return true if this pass uses separate scene blending */
+		bool hasSeparateSceneBlending() const;
+
         /** Retrieves the source blending factor for the material (as set using Materiall::setSceneBlending).
         */
         SceneBlendFactor getSourceBlendFactor() const;
@@ -558,6 +651,14 @@ namespace Ogre {
         */
         SceneBlendFactor getDestBlendFactor() const;
 
+	    /** Retrieves the alpha source blending factor for the material (as set using Materiall::setSeparateSceneBlending).
+        */
+		SceneBlendFactor getSourceBlendFactorAlpha() const;
+
+	    /** Retrieves the alpha destination blending factor for the material (as set using Materiall::setSeparateSceneBlending).
+        */
+		SceneBlendFactor getDestBlendFactorAlpha() const;
+
 		/** Returns true if this pass has some element of transparency. */
 		bool isTransparent(void) const;
 
@@ -646,7 +747,7 @@ namespace Ogre {
         CullingMode getCullingMode(void) const;
 
         /** Sets the manual culling mode, performed by CPU rather than hardware.
-        @pemarks
+        @remarks
         In some situations you want to use manual culling of triangles rather than sending the
         triangles to the hardware and letting it cull them. This setting only takes effect on SceneManager's
         that use it (since it is best used on large groups of planar world geometry rather than on movable
@@ -726,6 +827,23 @@ namespace Ogre {
 		*/
 		PolygonMode getPolygonMode(void) const;
 
+		/** Sets whether this pass's chosen detail level can be
+			overridden (downgraded) by the camera setting. 
+		@param override true means that a lower camera detail will override this
+			pass's detail level, false means it won't (default true).
+		*/
+		virtual void setPolygonModeOverrideable(bool override)
+		{
+			mPolygonModeOverrideable = override;
+		}
+
+		/** Gets whether this renderable's chosen detail level can be
+			overridden (downgraded) by the camera setting. 
+		*/
+		virtual bool getPolygonModeOverrideable(void) const
+		{
+			return mPolygonModeOverrideable;
+		}
         /** Sets the fogging mode applied to this pass.
         @remarks
         Fogging is an effect that is applied as polys are rendered. Sometimes, you want
@@ -737,7 +855,7 @@ namespace Ogre {
         lets you change the fog behaviour for this pass compared to the standard scene-level fog.
         @param
         overrideScene If true, you authorise this pass to override the scene's fog params with it's own settings.
-        If you specify false, so other parameters are necessary, and this is the default behaviour for passs.
+        If you specify false, so other parameters are necessary, and this is the default behaviour for passes.
         @param
         mode Only applicable if overrideScene is true. You can disable fog which is turned on for the
         rest of the scene by specifying FOG_NONE. Otherwise, set a pass-specific fog mode as
@@ -818,16 +936,28 @@ namespace Ogre {
         float getDepthBiasConstant(void) const;
 		/** Retrieves the slope-scale depth bias value as set by setDepthBias. */
 		float getDepthBiasSlopeScale(void) const;
+		/** Sets a factor which derives an additional depth bias from the number 
+			of times a pass is iterated.
+		@remarks
+			The Final depth bias will be the constant depth bias as set through
+			setDepthBias, plus this value times the iteration number. 
+		*/
+		void setIterationDepthBias(float biasPerIteration);
+		/** Gets a factor which derives an additional depth bias from the number 
+			of times a pass is iterated.
+		*/
+		float getIterationDepthBias() const;
 
         /** Sets the way the pass will have use alpha to totally reject pixels from the pipeline.
         @remarks
 			The default is CMPF_ALWAYS_PASS i.e. alpha is not used to reject pixels.
         @param func The comparison which must pass for the pixel to be written.
         @param value 1 byte value against which alpha values will be tested(0-255)
+		@param alphaToCoverageEnabled Whether to enable alpha to coverage support
         @note
 			This option applies in both the fixed function and the programmable pipeline.
         */
-        void setAlphaRejectSettings(CompareFunction func, unsigned char value);
+        void setAlphaRejectSettings(CompareFunction func, unsigned char value, bool alphaToCoverageEnabled = false);
 
 		/** Sets the alpha reject function. See setAlphaRejectSettings for more information.
 		*/
@@ -844,7 +974,39 @@ namespace Ogre {
         /** Gets the alpha reject value. See setAlphaRejectSettings for more information.
         */
 		unsigned char getAlphaRejectValue(void) const { return mAlphaRejectVal; }
-        /** Sets whether or not this pass should iterate per light or number of
+
+		/** Sets whether to use alpha to coverage (A2C) when blending alpha rejected values. 
+		@remarks
+			Alpha to coverage performs multisampling on the edges of alpha-rejected
+			textures to produce a smoother result. It is only supported when multisampling
+			is already enabled on the render target, and when the hardware supports
+			alpha to coverage (see RenderSystemCapabilities). 
+		*/
+		void setAlphaToCoverageEnabled(bool enabled);
+
+		/** Gets whether to use alpha to coverage (A2C) when blending alpha rejected values. 
+		*/
+		bool isAlphaToCoverageEnabled() const { return mAlphaToCoverageEnabled; }
+
+        /** Sets whether or not transparent sorting is enabled.
+        @param enabled
+			If false depth sorting of this material will be disabled.
+        @remarks
+			By default all transparent materials are sorted such that renderables furthest
+			away from the camera are rendered first. This is usually the desired behaviour
+			but in certain cases this depth sorting may be unnecessary and undesirable. If
+			for example it is necessary to ensure the rendering order does not change from
+			one frame to the next.
+		@note
+			This will have no effect on non-transparent materials.
+        */
+        void setTransparentSortingEnabled(bool enabled);
+
+        /** Returns whether or not transparent sorting is enabled.
+        */
+		bool getTransparentSortingEnabled(void) const;
+
+		/** Sets whether or not this pass should iterate per light or number of
 			lights which can affect the object being rendered.
 		@remarks
 			The default behaviour for a pass (when this option is 'false'), is
@@ -962,7 +1124,7 @@ namespace Ogre {
         with only the ambient colour being used (which the engine will ensure
         is bound to the shadow colour).
         @par
-        Therefore, it is up to implemetors of vertex programs to provide an
+        Therefore, it is up to implementors of vertex programs to provide an
         alternative vertex program which can be used to render the object
         to a shadow texture. Do all the same vertex transforms, but set the
         colour of the vertex to the ambient colour, as bound using the
@@ -977,7 +1139,7 @@ namespace Ogre {
         This is only applicable to programmable passes.
         @par
         The default behaviour is for Ogre to switch to fixed-function
-        rendering if an explict vertex program alternative is not set.
+        rendering if an explicit vertex program alternative is not set.
         */
         void setShadowCasterVertexProgram(const String& name);
         /** Sets the vertex program parameters for rendering as a shadow caster.
@@ -1006,7 +1168,7 @@ namespace Ogre {
             appropriate vertex transformation, but generates projective texture
             coordinates.
         @par
-            Therefore, it is up to implemetors of vertex programs to provide an
+            Therefore, it is up to implementors of vertex programs to provide an
             alternative vertex program which can be used to render the object
             as a shadow receiver. Do all the same vertex transforms, but generate
             <strong>2 sets</strong> of texture coordinates using the auto parameter
@@ -1103,6 +1265,35 @@ namespace Ogre {
 		/** Gets the fragment program used by this pass, only available after _load(). */
 		const GpuProgramPtr& getFragmentProgram(void) const;
 
+		/** Sets the details of the geometry program to use.
+		@remarks
+			Only applicable to programmable passes, this sets the details of
+			the geometry program to use in this pass. The program will not be
+			loaded until the parent Material is loaded.
+		@param name The name of the program - this must have been
+			created using GpuProgramManager by the time that this Pass
+			is loaded. If this parameter is blank, any geometry program in this pass is disabled.
+        @param resetParams
+            If true, this will create a fresh set of parameters from the
+            new program being linked, so if you had previously set parameters
+            you will have to set them again. If you set this to false, you must
+            be absolutely sure that the parameters match perfectly, and in the
+            case of named parameters refers to the indexes underlying them,
+            not just the names.
+		*/
+		void setGeometryProgram(const String& name, bool resetParams = true);
+		/** Sets the geometry program parameters.
+		@remarks
+			Only applicable to programmable passes.
+		*/
+		void setGeometryProgramParameters(GpuProgramParametersSharedPtr params);
+		/** Gets the name of the geometry program used by this pass. */
+		const String& getGeometryProgramName(void) const;
+		/** Gets the geometry program parameters used by this pass. */
+		GpuProgramParametersSharedPtr getGeometryProgramParameters(void) const;
+		/** Gets the geometry program used by this pass, only available after _load(). */
+		const GpuProgramPtr& getGeometryProgram(void) const;
+
 		/** Splits this Pass to one which can be handled in the number of
 			texture units specified.
 		@remarks
@@ -1119,6 +1310,10 @@ namespace Ogre {
 		/** Internal method to adjust pass index. */
 		void _notifyIndex(unsigned short index);
 
+		/** Internal method for preparing to load this pass. */
+		void _prepare(void);
+		/** Internal method for undoing the load preparartion for this pass. */
+		void _unprepare(void);
 		/** Internal method for loading this pass. */
 		void _load(void);
 		/** Internal method for unloading this pass. */
@@ -1146,9 +1341,9 @@ namespace Ogre {
         void _notifyNeedsRecompile(void);
 
         /** Update any automatic parameters (except lights) on this pass */
-        void _updateAutoParamsNoLights(const AutoParamDataSource& source) const;
+        void _updateAutoParamsNoLights(const AutoParamDataSource* source) const;
         /** Update any automatic light parameters on this pass */
-        void _updateAutoParamsLightsOnly(const AutoParamDataSource& source) const;
+        void _updateAutoParamsLightsOnly(const AutoParamDataSource* source) const;
 
 		/** Gets the 'nth' texture which references the given content type.
 		@remarks
@@ -1177,6 +1372,22 @@ namespace Ogre {
         @see TextureUnitState::setTextureAnisotropy
         */
         void setTextureAnisotropy(unsigned int maxAniso);
+		/** If set to true, this forces normals to be normalised dynamically 
+			by the hardware for this pass.
+		@remarks
+			This option can be used to prevent lighting variations when scaling an
+			object - normally because this scaling is hardware based, the normals 
+			get scaled too which causes lighting to become inconsistent. By default the
+			SceneManager detects scaled objects and does this for you, but 
+			this has an overhead so you might want to turn that off through
+			SceneManager::setNormaliseNormalsOnScale(false) and only do it per-Pass
+			when you need to.
+		*/
+		void setNormaliseNormals(bool normalise) { mNormaliseNormals = normalise; }
+
+		/** Returns true if this pass has auto-normalisation of normals set. */
+		bool getNormaliseNormals(void) const {return mNormaliseNormals; }
+
 		/** Static method to retrieve all the Passes which need their
 		    hash values recalculated.
 		*/
@@ -1237,7 +1448,85 @@ namespace Ogre {
         */
         bool applyTextureAliases(const AliasTextureNamePairList& aliasList, const bool apply = true) const;
 
+		/** Sets whether or not this pass will be clipped by a scissor rectangle
+			encompassing the lights that are being used in it.
+		@remarks
+			In order to cut down on fillrate when you have a number of fixed-range
+			lights in the scene, you can enable this option to request that
+			during rendering, only the region of the screen which is covered by
+			the lights is rendered. This region is the screen-space rectangle 
+			covering the union of the spheres making up the light ranges. Directional
+			lights are ignored for this.
+		@par
+			This is only likely to be useful for multipass additive lighting 
+			algorithms, where the scene has already been 'seeded' with an ambient 
+			pass and this pass is just adding light in affected areas.
+		@note
+			When using SHADOWTYPE_STENCIL_ADDITIVE or SHADOWTYPE_TEXTURE_ADDITIVE,
+			this option is implicitly used for all per-light passes and does
+			not need to be specified. If you are not using shadows or are using
+			a modulative or an integrated shadow technique then this could be useful.
 
+		*/
+		void setLightScissoringEnabled(bool enabled) { mLightScissoring = enabled; }
+		/** Gets whether or not this pass will be clipped by a scissor rectangle
+			encompassing the lights that are being used in it.
+		*/
+		bool getLightScissoringEnabled() const { return mLightScissoring; }
+
+		/** Gets whether or not this pass will be clipped by user clips planes
+			bounding the area covered by the light.
+		@remarks
+			In order to cut down on the geometry set up to render this pass 
+			when you have a single fixed-range light being rendered through it, 
+			you can enable this option to request that during triangle setup, 
+			clip planes are defined to bound the range of the light. In the case
+			of a point light these planes form a cube, and in the case of 
+			a spotlight they form a pyramid. Directional lights are never clipped.
+		@par
+			This option is only likely to be useful for multipass additive lighting 
+			algorithms, where the scene has already been 'seeded' with an ambient 
+			pass and this pass is just adding light in affected areas. In addition,
+			it will only be honoured if there is exactly one non-directional light
+			being used in this pass. Also, these clip planes override any user clip
+			planes set on Camera.
+		@note
+			When using SHADOWTYPE_STENCIL_ADDITIVE or SHADOWTYPE_TEXTURE_ADDITIVE,
+			this option is automatically used for all per-light passes if you 
+			enable SceneManager::setShadowUseLightClipPlanes and does
+			not need to be specified. It is disabled by default since clip planes have
+			a cost of their own which may not always exceed the benefits they give you.
+		*/
+		void setLightClipPlanesEnabled(bool enabled) { mLightClipPlanes = enabled; }
+		/** Gets whether or not this pass will be clipped by user clips planes
+			bounding the area covered by the light.
+		*/
+		bool getLightClipPlanesEnabled() const { return mLightClipPlanes; }
+
+		/** Manually set which illumination stage this pass is a member of.
+		@remarks
+			When using an additive lighting mode (SHADOWTYPE_STENCIL_ADDITIVE or
+			SHADOWTYPE_TEXTURE_ADDITIVE), the scene is rendered in 3 discrete
+			stages, ambient (or pre-lighting), per-light (once per light, with 
+			shadowing) and decal (or post-lighting). Usually OGRE figures out how
+			to categorise your passes automatically, but there are some effects you
+			cannot achieve without manually controlling the illumination. For example
+			specular effects are muted by the typical sequence because all textures
+			are saved until the IS_DECAL stage which mutes the specular effect. 
+			Instead, you could do texturing within the per-light stage if it's
+			possible for your material and thus add the specular on after the
+			decal texturing, and have no post-light rendering. 
+		@par
+			If you assign an illumination stage to a pass you have to assign it
+			to all passes in the technique otherwise it will be ignored. Also note
+			that whilst you can have more than one pass in each group, they cannot
+			alternate, ie all ambient passes will be before all per-light passes, 
+			which will also be before all decal passes. Within their categories
+			the passes will retain their ordering though.
+		*/
+		void setIlluminationStage(IlluminationStage is) { mIlluminationStage = is; }
+		/// Get the manually assigned illumination stage, if any
+		IlluminationStage getIlluminationStage() const { return mIlluminationStage; }
 		/** There are some default hash functions used to order passes so that
 			render state changes are minimised, this enumerates them.
 		*/
@@ -1284,15 +1573,6 @@ namespace Ogre {
         
     };
 
-    enum IlluminationStage
-    {
-        /// Part of the rendering which occurs without any kind of direct lighting
-        IS_AMBIENT,
-        /// Part of the rendering which occurs per light
-        IS_PER_LIGHT,
-        /// Post-lighting rendering
-        IS_DECAL
-    };
     /** Struct recording a pass which can be used for a specific illumination stage.
     @remarks
         This structure is used to record categorised passes which fit into a
@@ -1302,7 +1582,7 @@ namespace Ogre {
         may require splitting into its component parts in order to be categorised
         properly.
     */
-    struct IlluminationPass
+	struct IlluminationPass : public PassAlloc
     {
         IlluminationStage stage;
         /// The pass to use in this stage
@@ -1311,6 +1591,8 @@ namespace Ogre {
         bool destroyOnShutdown;
         /// The original pass which spawned this one
         Pass* originalPass;
+
+		IlluminationPass() {}
     };
 
     typedef std::vector<IlluminationPass*> IlluminationPassList;
