@@ -34,6 +34,8 @@ Torus Knot Software Ltd.
 #include "OgreBlendMode.h"
 #include "OgreCommon.h"
 #include "OgrePass.h"
+#include "OgreIteratorWrappers.h"
+#include "OgreRenderSystemCapabilities.h"
 
 namespace Ogre {
     /** Class representing an approach to rendering this particular Material. 
@@ -42,7 +44,7 @@ namespace Ogre {
         unless you specifically request a lower detail technique (say for distant
         rendering).
     */
-    class _OgreExport Technique
+	class _OgreExport Technique : public TechniqueAlloc
     {
     protected:
         // illumination pass state type
@@ -71,7 +73,69 @@ namespace Ogre {
 
         /// Internal method for clearing illumination pass list
         void clearIlluminationPasses(void);
-    public:
+		/// Internal method - check for manually assigned illumination passes
+		bool checkManuallyOrganisedIlluminationPasses();
+
+
+		/** When casting shadow, if not using default Ogre shadow casting material, or 
+		* nor using fixed function casting, mShadowCasterMaterial let you customize per material
+		* shadow caster behavior
+		*/
+		MaterialPtr mShadowCasterMaterial;
+		/** When casting shadow, if not using default Ogre shadow casting material, or 
+		* nor using fixed function casting, mShadowCasterMaterial let you customize per material
+		* shadow caster behavior.There only material name is stored so that it can be loaded once all file parsed in a resource group.
+		*/
+		String mShadowCasterMaterialName;
+		/** When receiving shadow, if not using default Ogre shadow receiving material, or 
+		* nor using fixed function texture projection receiving, mShadowReceiverMaterial let you customize per material
+		* shadow caster behavior
+		*/
+		MaterialPtr mShadowReceiverMaterial;
+		/** When receiving shadow, if not using default Ogre shadow receiving material, or 
+		* nor using fixed function texture projection receiving, mShadowReceiverMaterial let you customize per material
+		* shadow caster behavior. There only material name is stored so that it can be loaded once all file parsed in a resource group.
+		*/
+		String mShadowReceiverMaterialName;	
+
+	public:
+		/** Directive used to manually control technique support based on the
+			inclusion or exclusion of some factor.
+		*/
+		enum IncludeOrExclude
+		{
+			/// Inclusive - only support if present
+			INCLUDE = 0,
+			/// Exclusive - do not support if present
+			EXCLUDE = 1
+		};
+		/// Rule controlling whether technique is deemed supported based on GPU vendor
+		struct GPUVendorRule
+		{
+			GPUVendor vendor;
+			IncludeOrExclude includeOrExclude;
+			GPUVendorRule()
+				: vendor(GPU_UNKNOWN), includeOrExclude(EXCLUDE) {}
+			GPUVendorRule(GPUVendor v, IncludeOrExclude ie)
+				: vendor(v), includeOrExclude(ie) {}
+		};
+		/// Rule controlling whether technique is deemed supported based on GPU device name
+		struct GPUDeviceNameRule
+		{
+			String devicePattern;
+			IncludeOrExclude includeOrExclude;
+			bool caseSensitive;
+			GPUDeviceNameRule()
+				: includeOrExclude(EXCLUDE), caseSensitive(false) {}
+			GPUDeviceNameRule(const String& pattern, IncludeOrExclude ie, bool caseSen)
+				: devicePattern(pattern), includeOrExclude(ie), caseSensitive(caseSen) {}
+		};
+		typedef std::vector<GPUVendorRule> GPUVendorRuleList;
+		typedef std::vector<GPUDeviceNameRule> GPUDeviceNameRuleList;
+	protected:
+		GPUVendorRuleList mGPUVendorRules;
+		GPUDeviceNameRuleList mGPUDeviceNameRules;
+	public:
         /// Constructor
         Technique(Material* parent);
         /// Copy constructor
@@ -87,13 +151,17 @@ namespace Ogre {
 		@returns Any information explaining problems with the compile.
 		*/
         String _compile(bool autoManageTextureUnits);
+		/// Internal method for checking GPU vendor / device rules
+		bool checkGPURules(StringUtil::StrStreamType& errors);
+		/// Internal method for checking hardware support
+		bool checkHardwareSupport(bool autoManageTextureUnits, StringUtil::StrStreamType& compileErrors);
         /** Internal method for splitting the passes into illumination passes. */        
         void _compileIlluminationPasses(void);
 
 
         /** Creates a new Pass for this Technique.
         @remarks
-            A Pass is a single rendering pass, ie a single draw of the given material.
+            A Pass is a single rendering pass, i.e. a single draw of the given material.
             Note that if you create a pass without a fragment program, during compilation of the
             material the pass may be split into multiple passes if the graphics card cannot
             handle the number of texture units requested. For passes with fragment programs, however, 
@@ -143,6 +211,17 @@ namespace Ogre {
 		*/
 		bool isTransparent(void) const;
 
+		/** Returns true if this Technique has transparent sorting enabled. 
+		@remarks
+			This basically boils down to whether the first pass
+			has transparent sorting enabled or not
+		*/
+		bool isTransparentSortingEnabled(void) const;
+
+        /** Internal prepare method, derived from call to Material::prepare. */
+        void _prepare(void);
+        /** Internal unprepare method, derived from call to Material::unprepare. */
+        void _unprepare(void);
         /** Internal load method, derived from call to Material::load. */
         void _load(void);
         /** Internal unload method, derived from call to Material::unload. */
@@ -154,6 +233,24 @@ namespace Ogre {
         /** Tells the technique that it needs recompilation. */
         void _notifyNeedsRecompile(void);
 
+		/** return this material specific  shadow casting specific material
+		*/
+		Ogre::MaterialPtr getShadowCasterMaterial() const;
+		/** set this material specific  shadow casting specific material
+		*/
+		void setShadowCasterMaterial(Ogre::MaterialPtr val);
+		/** set this material specific  shadow casting specific material
+		*/
+		void setShadowCasterMaterial(const Ogre::String &name);
+		/** return this material specific shadow receiving specific material
+		*/
+		Ogre::MaterialPtr getShadowReceiverMaterial() const;
+		/** set this material specific  shadow receiving specific material
+		*/
+		void setShadowReceiverMaterial(Ogre::MaterialPtr val);
+		/** set this material specific  shadow receiving specific material
+		*/
+		void setShadowReceiverMaterial(const Ogre::String &name);
 
         // -------------------------------------------------------------------------------
         // The following methods are to make migration from previous versions simpler
@@ -393,6 +490,16 @@ namespace Ogre {
         */
         void setSceneBlending( const SceneBlendType sbt );
 
+        /** Sets the kind of blending every pass has with the existing contents of the scene, using individual factors both color and alpha channels
+        @note
+            This property actually exists on the Pass class. For simplicity, this method allows 
+            you to set these properties for every current Pass within this Technique. If 
+            you need more precision, retrieve the Pass instance and set the
+            property there.
+        @see Pass::setSeparateSceneBlending
+        */
+        void setSeparateSceneBlending( const SceneBlendType sbt, const SceneBlendType sbta );
+
         /** Allows very fine control of blending every Pass with the existing contents of the scene.
         @note
             This property actually exists on the Pass class. For simplicity, this method allows 
@@ -403,6 +510,16 @@ namespace Ogre {
         */
         void setSceneBlending( const SceneBlendFactor sourceFactor, const SceneBlendFactor destFactor);
 
+        /** Allows very fine control of blending every Pass with the existing contents of the scene, using individual factors both color and alpha channels
+        @note
+            This property actually exists on the Pass class. For simplicity, this method allows 
+            you to set these properties for every current Pass within this Technique. If 
+            you need more precision, retrieve the Pass instance and set the
+            property there.
+        @see Pass::setSeparateSceneBlending
+        */
+        void setSeparateSceneBlending( const SceneBlendFactor sourceFactor, const SceneBlendFactor destFactor, const SceneBlendFactor sourceFactorAlpha, const SceneBlendFactor destFactorAlpha);
+
         /** Assigns a level-of-detail (LOD) index to this Technique.
         @remarks
             As noted previously, as well as providing fallback support for various
@@ -460,8 +577,8 @@ namespace Ogre {
 
         /** Set the name of the technique.
         @remarks
-        The use of technique name is optional.  Its usefull in material scripts where a material could inherit
-        from another material and only want to modify a particalar technique.
+		The use of technique name is optional.  Its useful in material scripts where a material could inherit
+		from another material and only want to modify a particular technique.
         */
         void setName(const String& name);
         /// Gets the name of the technique
@@ -481,6 +598,85 @@ namespace Ogre {
         bool applyTextureAliases(const AliasTextureNamePairList& aliasList, const bool apply = true) const;
 
 
+		/** Add a rule which manually influences the support for this technique based
+			on a GPU vendor.
+		@remarks
+			You can use this facility to manually control whether a technique is
+			considered supported, based on a GPU vendor. You can add inclusive
+			or exclusive rules, and you can add as many of each as you like. If
+			at least one inclusive rule is added, a	technique is considered 
+			unsupported if it does not match any of those inclusive rules. If exclusive rules are
+			added, the technique is considered unsupported if it matches any of
+			those inclusive rules. 
+		@note
+			Any rule for the same vendor will be removed before adding this one.
+		@param vendor The GPU vendor
+		@param includeOrExclude Whether this is an inclusive or exclusive rule
+		*/
+		void addGPUVendorRule(GPUVendor vendor, IncludeOrExclude includeOrExclude);
+		/** Add a rule which manually influences the support for this technique based
+			on a GPU vendor.
+		@remarks
+			You can use this facility to manually control whether a technique is
+			considered supported, based on a GPU vendor. You can add inclusive
+			or exclusive rules, and you can add as many of each as you like. If
+			at least one inclusive rule is added, a	technique is considered 
+			unsupported if it does not match any of those inclusive rules. If exclusive rules are
+			added, the technique is considered unsupported if it matches any of
+			those inclusive rules. 
+		@note
+			Any rule for the same vendor will be removed before adding this one.
+		*/
+		void addGPUVendorRule(const GPUVendorRule& rule);
+		/** Removes a matching vendor rule.
+		@see addGPUVendorRule
+		*/
+		void removeGPUVendorRule(GPUVendor vendor);
+		typedef ConstVectorIterator<GPUVendorRuleList> GPUVendorRuleIterator;
+		/// Get an iterator over the currently registered vendor rules.
+		GPUVendorRuleIterator getGPUVendorRuleIterator() const;
+
+		/** Add a rule which manually influences the support for this technique based
+			on a pattern that matches a GPU device name (e.g. '*8800*').
+		@remarks
+			You can use this facility to manually control whether a technique is
+			considered supported, based on a GPU device name pattern. You can add inclusive
+			or exclusive rules, and you can add as many of each as you like. If
+			at least one inclusive rule is added, a	technique is considered 
+			unsupported if it does not match any of those inclusive rules. If exclusive rules are
+			added, the technique is considered unsupported if it matches any of
+			those inclusive rules. The pattern you supply can include wildcard
+			characters ('*') if you only want to match part of the device name.
+		@note
+			Any rule for the same device pattern will be removed before adding this one.
+		@param devicePattern The GPU vendor
+		@param includeOrExclude Whether this is an inclusive or exclusive rule
+		@param caseSensitive Whether the match is case sensitive or not
+		*/
+		void addGPUDeviceNameRule(const String& devicePattern, IncludeOrExclude includeOrExclude, bool caseSensitive = false);
+		/** Add a rule which manually influences the support for this technique based
+			on a pattern that matches a GPU device name (e.g. '*8800*').
+		@remarks
+			You can use this facility to manually control whether a technique is
+			considered supported, based on a GPU device name pattern. You can add inclusive
+			or exclusive rules, and you can add as many of each as you like. If
+			at least one inclusive rule is added, a	technique is considered 
+			unsupported if it does not match any of those inclusive rules. If exclusive rules are
+			added, the technique is considered unsupported if it matches any of
+			those inclusive rules. The pattern you supply can include wildcard
+			characters ('*') if you only want to match part of the device name.
+		@note
+			Any rule for the same device pattern will be removed before adding this one.
+		*/
+		void addGPUDeviceNameRule(const GPUDeviceNameRule& rule);
+		/** Removes a matching device name rule.
+		@see addGPUDeviceNameRule
+		*/
+		void removeGPUDeviceNameRule(const String& devicePattern);
+		typedef ConstVectorIterator<GPUDeviceNameRuleList> GPUDeviceNameRuleIterator;
+		/// Get an iterator over the currently registered device name rules.
+		GPUDeviceNameRuleIterator getGPUDeviceNameRuleIterator() const;
+
     };
 
 
