@@ -75,8 +75,8 @@ namespace Ogre {
 		mRadius(0.0f),
 		mTexCoordDir(TCD_U)
 	{
-		mVertexData = new VertexData();
-		mIndexData = new IndexData();
+		mVertexData = OGRE_NEW VertexData();
+		mIndexData = OGRE_NEW IndexData();
 
 		mOtherTexCoordRange[0] = 0.0f;
 		mOtherTexCoordRange[1] = 1.0f;
@@ -92,8 +92,8 @@ namespace Ogre {
 	//-----------------------------------------------------------------------
 	BillboardChain::~BillboardChain()
 	{
-		delete mVertexData;
-		delete mIndexData;
+		OGRE_DELETE mVertexData;
+		OGRE_DELETE mIndexData;
 	}
 	//-----------------------------------------------------------------------
 	void BillboardChain::setupChainContainers(void)
@@ -319,7 +319,7 @@ namespace Ogre {
 		{
 			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND,
 				"chainIndex out of bounds",
-				"BillboardChain::removeChainElement");
+				"BillboardChain::clearChain");
 		}
 		ChainSegment& seg = mChainSegmentList[chainIndex];
 
@@ -383,7 +383,7 @@ namespace Ogre {
 		{
 			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND,
 				"chainIndex out of bounds",
-				"BillboardChain::updateChainElement");
+				"BillboardChain::getChainElement");
 		}
 		const ChainSegment& seg = mChainSegmentList[chainIndex];
 
@@ -394,6 +394,19 @@ namespace Ogre {
 		return mChainElementList[idx];
 	}
 	//-----------------------------------------------------------------------
+	size_t BillboardChain::getNumChainElements(size_t chainIndex) const
+	{
+		if (chainIndex >= mChainCount)
+		{
+			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND,
+				"chainIndex out of bounds",
+				"BillboardChain::getNumChainElements");
+		}
+		const ChainSegment& seg = mChainSegmentList[chainIndex];
+
+		return seg.tail - seg.head + 1;
+	}
+	//-----------------------------------------------------------------------
 	void BillboardChain::updateBoundingBox(void) const
 	{
 		if (mBoundsDirty)
@@ -737,20 +750,17 @@ namespace Ogre {
 		*xform = _getParentNodeFullTransform();
 	}
 	//-----------------------------------------------------------------------
-	const Quaternion& BillboardChain::getWorldOrientation(void) const
-	{
-		return getParentNode()->_getDerivedOrientation();
-	}
-	//-----------------------------------------------------------------------
-	const Vector3& BillboardChain::getWorldPosition(void) const
-	{
-		return getParentNode()->_getDerivedPosition();
-	}
-	//-----------------------------------------------------------------------
 	const LightList& BillboardChain::getLights(void) const
 	{
 		return queryLights();
 	}
+	//---------------------------------------------------------------------
+	void BillboardChain::visitRenderables(Renderable::Visitor* visitor, 
+		bool debugRenderables)
+	{
+		// only one renderable
+		visitor->visit(this, 0, false);
+	}
 	//-----------------------------------------------------------------------
 	//-----------------------------------------------------------------------
 	String BillboardChainFactory::FACTORY_TYPE_NAME = "BillboardChain";
@@ -799,13 +809,13 @@ namespace Ogre {
 
 		}
 
-		return new BillboardChain(name, maxElements, numberOfChains, useTex, useCol, dynamic);
+		return OGRE_NEW BillboardChain(name, maxElements, numberOfChains, useTex, useCol, dynamic);
 
 	}
 	//-----------------------------------------------------------------------
 	void BillboardChainFactory::destroyInstance( MovableObject* obj)
 	{
-		delete obj;
+		OGRE_DELETE  obj;
 	}
 
 }
