@@ -159,8 +159,7 @@ namespace Ogre {
         mAABB.merge(newMin);
         mAABB.merge(newMax);
 
-		Real sqlen = std::max(newMin.squaredLength(), newMax.squaredLength());
-		mBoundingRadius = std::max(mBoundingRadius, Math::Sqrt(sqlen));
+		mBoundingRadius = Math::boundingRadiusFromAABB(mAABB);
 
         return newBill;
     }
@@ -559,9 +558,16 @@ namespace Ogre {
             ActiveBillboardList::iterator i, iend;
 
             iend = mActiveBillboards.end();
+			Matrix4 invWorld;
+			if (mWorldSpace && getParentSceneNode())
+				invWorld = getParentSceneNode()->_getFullTransform().inverse();
+
             for (i = mActiveBillboards.begin(); i != iend; ++i)
             {
-                const Vector3& pos = (*i)->getPosition();
+                Vector3 pos = (*i)->getPosition();
+				// transform from world space to local space
+				if (mWorldSpace && getParentSceneNode())
+					pos = invWorld * pos;
                 min.makeFloor(pos);
                 max.makeCeil(pos);
 
