@@ -58,7 +58,8 @@ namespace Ogre {
 		mLastViewport(0),
 		mAutoAspectRatio(false),
 		mCullFrustum(0),
-		mUseRenderingDistance(true)
+		mUseRenderingDistance(true),
+		mLodCamera(0)
 
     {
 
@@ -310,7 +311,12 @@ namespace Ogre {
     void Camera::rotate(const Quaternion& q)
     {
         // Note the order of the mult, i.e. q comes after
-        mOrientation = q * mOrientation;
+
+		// Normalise the quat to avoid cumulative problems with precision
+		Quaternion qnorm = q;
+		qnorm.normalise();
+        mOrientation = qnorm * mOrientation;
+
         invalidateView();
 
     }
@@ -458,6 +464,7 @@ namespace Ogre {
     void Camera::setOrientation(const Quaternion& q)
     {
         mOrientation = q;
+		mOrientation.normalise();
         invalidateView();
     }
     //-----------------------------------------------------------------------
@@ -568,8 +575,28 @@ namespace Ogre {
 	{
 		return mSceneLodFactorInv;
 	}
+	//-----------------------------------------------------------------------
+	void Camera::setLodCamera(const Camera* lodCam)
+	{
+		if (lodCam == this)
+			mLodCamera = 0;
+		else
+			mLodCamera = lodCam;
+	}
+	//---------------------------------------------------------------------
+	const Camera* Camera::getLodCamera() const
+	{
+		return mLodCamera? mLodCamera : this;
+	}
     //-----------------------------------------------------------------------
-    Ray Camera::getCameraToViewportRay(Real screenX, Real screenY) const
+	Ray Camera::getCameraToViewportRay(Real screenX, Real screenY) const
+	{
+		Ray ret;
+		getCameraToViewportRay(screenX, screenY, &ret);
+		return ret;
+	}
+	//---------------------------------------------------------------------
+    void Camera::getCameraToViewportRay(Real screenX, Real screenY, Ray* outRay) const
     {
 		Matrix4 inverseVP = (getProjectionMatrix() * getViewMatrix(true)).inverse();
 
@@ -588,9 +615,87 @@ namespace Ogre {
 		Vector3 rayDirection = rayTarget - rayOrigin;
 		rayDirection.normalise();
 
-		return Ray(rayOrigin, rayDirection);
+		outRay->setOrigin(rayOrigin);
+		outRay->setDirection(rayDirection);
     } 
+	//---------------------------------------------------------------------
+	PlaneBoundedVolume Camera::getCameraToViewportBoxVolume(Real screenLeft, 
+		Real screenTop, Real screenRight, Real screenBottom, bool includeFarPlane)
+	{
+		PlaneBoundedVolume vol;
+		getCameraToViewportBoxVolume(screenLeft, screenTop, screenRight, screenBottom, 
+			&vol, includeFarPlane);
+		return vol;
 
+	}
+	//---------------------------------------------------------------------()
+	void Camera::getCameraToViewportBoxVolume(Real screenLeft, 
+		Real screenTop, Real screenRight, Real screenBottom, 
+		PlaneBoundedVolume* outVolume, bool includeFarPlane)
+	{
+		outVolume->planes.clear();
+
+		if (mProjType == PT_PERSPECTIVE)
+		{
+
+			// Use the corner rays to generate planes
+			Ray ul = getCameraToViewportRay(screenLeft, screenTop);
+			Ray ur = getCameraToViewportRay(screenRight, screenTop);
+			Ray bl = getCameraToViewportRay(screenLeft, screenBottom);
+			Ray br = getCameraToViewportRay(screenRight, screenBottom);
+
+
+			Vector3 normal;
+			// top plane
+			normal = ul.getDirection().crossProduct(ur.getDirection());
+			normal.normalise();
+			outVolume->planes.push_back(
+				Plane(normal, getDerivedPosition()));
+
+			// right plane
+			normal = ur.getDirection().crossProduct(br.getDirection());
+			normal.normalise();
+			outVolume->planes.push_back(
+				Plane(normal, getDerivedPosition()));
+
+			// bottom plane
+			normal = br.getDirection().crossProduct(bl.getDirection());
+			normal.normalise();
+			outVolume->planes.push_back(
+				Plane(normal, getDerivedPosition()));
+
+			// left plane
+			normal = bl.getDirection().crossProduct(ul.getDirection());
+			normal.normalise();
+			outVolume->planes.push_back(
+				Plane(normal, getDerivedPosition()));
+
+		}
+		else
+		{
+			// ortho planes are parallel to frustum planes
+
+			Ray ul = getCameraToViewportRay(screenLeft, screenTop);
+			Ray br = getCameraToViewportRay(screenRight, screenBottom);
+
+			updateFrustumPlanes();
+			outVolume->planes.push_back(
+				Plane(mFrustumPlanes[FRUSTUM_PLANE_TOP].normal, ul.getOrigin()));
+			outVolume->planes.push_back(
+				Plane(mFrustumPlanes[FRUSTUM_PLANE_RIGHT].normal, br.getOrigin()));
+			outVolume->planes.push_back(
+				Plane(mFrustumPlanes[FRUSTUM_PLANE_BOTTOM].normal, br.getOrigin()));
+			outVolume->planes.push_back(
+				Plane(mFrustumPlanes[FRUSTUM_PLANE_LEFT].normal, ul.getOrigin()));
+			
+
+		}
+
+		// near & far plane applicable to both projection types
+		outVolume->planes.push_back(getFrustumPlane(FRUSTUM_PLANE_NEAR));
+		if (includeFarPlane)
+			outVolume->planes.push_back(getFrustumPlane(FRUSTUM_PLANE_FAR));
+	}
     // -------------------------------------------------------------------
     void Camera::setWindow (Real Left, Real Top, Real Right, Real Bottom)
     {
@@ -957,6 +1062,20 @@ namespace Ogre {
 		}
 	}
 	//-----------------------------------------------------------------------
+	void Camera::synchroniseBaseSettingsWith(const Camera* cam)
+	{
+		this->setPosition(cam->getPosition());
+		this->setProjectionType(cam->getProjectionType());
+		this->setOrientation(cam->getOrientation());
+		this->setAspectRatio(cam->getAspectRatio());
+		this->setNearClipDistance(cam->getNearClipDistance());
+		this->setFarClipDistance(cam->getFarClipDistance());
+		this->setLodCamera(cam->getLodCamera());
+		this->setUseRenderingDistance(cam->getUseRenderingDistance());
+		this->setCullingFrustum(cam->getCullingFrustum());
+
+
+	}
 
 
 } // namespace Ogre
