@@ -201,7 +201,7 @@ namespace Ogre {
 		// this can happen if Mesh is loaded in background or after reinitialisation
 		if( mParentNode )
 		{
-			getParentSceneNode()->_updateBounds();
+			getParentSceneNode()->needUpdate();
 		}
 
 		mInitialised = true;
@@ -431,10 +431,15 @@ namespace Ogre {
     const AxisAlignedBox& Entity::getBoundingBox(void) const
     {
 		// Get from Mesh
-        mFullBoundingBox = mMesh->getBounds();
-        mFullBoundingBox.merge(getChildObjectsBoundingBox());
+		if (mMesh->isLoaded())
+		{
+			mFullBoundingBox = mMesh->getBounds();
+			mFullBoundingBox.merge(getChildObjectsBoundingBox());
 
-        // Don't scale here, this is taken into account when world BBox calculation is done
+			// Don't scale here, this is taken into account when world BBox calculation is done
+		}
+		else
+			mFullBoundingBox.setNull();
 
         return mFullBoundingBox;
     }
