@@ -58,7 +58,7 @@ namespace Ogre
     }
     //-----------------------------------------------------------------------
     MeshManager::MeshManager():
-    mBoundsPaddingFactor(0.01)
+    mBoundsPaddingFactor(0.01), mListener(0)
     {
         mPrepAllMeshesForShadowVolumes = false;
 
@@ -82,12 +82,16 @@ namespace Ogre
 		createPrefabSphere();
     }
     //-----------------------------------------------------------------------
-    MeshPtr MeshManager::load( const String& filename, const String& groupName, 
+    MeshManager::ResourceCreateOrRetrieveResult MeshManager::createOrRetrieve(
+        const String& name, const String& group,
+        bool isManual, ManualResourceLoader* loader,
+        const NameValuePairList* params,
 		HardwareBuffer::Usage vertexBufferUsage, 
 		HardwareBuffer::Usage indexBufferUsage, 
 		bool vertexBufferShadowed, bool indexBufferShadowed)
     {
-        ResourceCreateOrRetrieveResult res = createOrRetrieve(filename, groupName);
+        ResourceCreateOrRetrieveResult res = 
+            ResourceManager::createOrRetrieve(name,group,isManual,loader,params);
 		MeshPtr pMesh = res.first;
 		// Was it created?
         if (res.second)
@@ -95,9 +99,32 @@ namespace Ogre
 			pMesh->setVertexBufferPolicy(vertexBufferUsage, vertexBufferShadowed);
 			pMesh->setIndexBufferPolicy(indexBufferUsage, indexBufferShadowed);
         }
+        return res;
+
+    }
+    //-----------------------------------------------------------------------
+    MeshPtr MeshManager::prepare( const String& filename, const String& groupName, 
+		HardwareBuffer::Usage vertexBufferUsage, 
+		HardwareBuffer::Usage indexBufferUsage, 
+		bool vertexBufferShadowed, bool indexBufferShadowed)
+    {
+		MeshPtr pMesh = createOrRetrieve(filename,groupName,false,0,0,
+                                         vertexBufferUsage,indexBufferUsage,
+                                         vertexBufferShadowed,indexBufferShadowed).first;
+		pMesh->prepare();
+        return pMesh;
+    }
+    //-----------------------------------------------------------------------
+    MeshPtr MeshManager::load( const String& filename, const String& groupName, 
+		HardwareBuffer::Usage vertexBufferUsage, 
+		HardwareBuffer::Usage indexBufferUsage, 
+		bool vertexBufferShadowed, bool indexBufferShadowed)
+    {
+		MeshPtr pMesh = createOrRetrieve(filename,groupName,false,0,0,
+                                         vertexBufferUsage,indexBufferUsage,
+                                         vertexBufferShadowed,indexBufferShadowed).first;
 		pMesh->load();
         return pMesh;
-
     }
     //-----------------------------------------------------------------------
     MeshPtr MeshManager::createManual( const String& name, const String& groupName, 
@@ -335,7 +362,7 @@ namespace Ogre
 		// to preserve previous behaviour, load immediately
 		msh->load();
 	}
-
+	//-------------------------------------------------------------------------
 	void MeshManager::createPrefabSphere(void)
 	{
 		MeshPtr msh = create(
@@ -347,7 +374,16 @@ namespace Ogre
 		// to preserve previous behaviour, load immediately
 		msh->load();
 	}
-
+	//-------------------------------------------------------------------------
+	void MeshManager::setListener(Ogre::MeshSerializerListener *listener)
+	{
+		mListener = listener;
+	}
+	//-------------------------------------------------------------------------
+	MeshSerializerListener *MeshManager::getListener()
+	{
+		return mListener;
+	}
     //-----------------------------------------------------------------------
 	void MeshManager::loadResource(Resource* res)
 	{
@@ -397,7 +433,7 @@ namespace Ogre
 
         // Set up vertex data
         // Use a single shared buffer
-        pMesh->sharedVertexData = new VertexData();
+        pMesh->sharedVertexData = OGRE_NEW VertexData();
         VertexData* vertexData = pMesh->sharedVertexData;
         // Set up Vertex Declaration
         VertexDeclaration* vertexDecl = vertexData->vertexDeclaration;
@@ -543,7 +579,7 @@ namespace Ogre
         SubMesh *pSub = pMesh->createSubMesh();
 
         // Set options
-        pMesh->sharedVertexData = new VertexData();
+        pMesh->sharedVertexData = OGRE_NEW VertexData();
         pMesh->sharedVertexData->vertexStart = 0;
         VertexBufferBinding* bind = pMesh->sharedVertexData->vertexBufferBinding;
         VertexDeclaration* decl = pMesh->sharedVertexData->vertexDeclaration;
@@ -700,7 +736,7 @@ namespace Ogre
 
         // Set up vertex data
         // Use a single shared buffer
-        pMesh->sharedVertexData = new VertexData();
+        pMesh->sharedVertexData = OGRE_NEW VertexData();
         VertexData* vertexData = pMesh->sharedVertexData;
         // Set up Vertex Declaration
         VertexDeclaration* vertexDecl = vertexData->vertexDeclaration;
@@ -891,7 +927,7 @@ namespace Ogre
             OGRE_EXCEPT(Exception::ERR_DUPLICATE_ITEM, "A mesh called " + name + 
                 " already exists!", "MeshManager::createBezierPatch");
         }
-        PatchMesh* pm = new PatchMesh(this, name, getNextHandle(), groupName);
+        PatchMesh* pm = OGRE_NEW PatchMesh(this, name, getNextHandle(), groupName);
         pm->define(controlPointBuffer, declaration, width, height,
             uMaxSubdivisionLevel, vMaxSubdivisionLevel, visibleSide, vbUsage, ibUsage,
             vbUseShadow, ibUseShadow);
@@ -927,7 +963,7 @@ namespace Ogre
         const NameValuePairList* createParams)
     {
         // no use for createParams here
-        return new Mesh(this, name, handle, group, isManual, loader);
+        return OGRE_NEW Mesh(this, name, handle, group, isManual, loader);
     }
     //-----------------------------------------------------------------------
 
