@@ -131,7 +131,7 @@ namespace Ogre {
 		mNonvisibleTimeout(0),
 		mNonvisibleTimeoutSet(false),
 		mTimeSinceLastVisible(0),
-		mLastVisibleFrame(Root::getSingleton().getCurrentFrameNumber()),
+		mLastVisibleFrame(Root::getSingleton().getNextFrameNumber()),
         mTimeController(0),
 		mEmittedEmitterPoolInitialised(false),
         mRenderer(0), 
@@ -170,7 +170,7 @@ namespace Ogre {
         ParticlePool::iterator i;
         for (i = mParticlePool.begin(); i != mParticlePool.end(); ++i)
         {
-            delete *i;
+            OGRE_DELETE *i;
         }
 
         if (mRenderer)
@@ -373,7 +373,7 @@ namespace Ogre {
 		{
 			// Check whether it's been more than one frame (update is ahead of
 			// camera notification by one frame because of the ordering)
-			long frameDiff = Root::getSingleton().getCurrentFrameNumber() - mLastVisibleFrame;
+			long frameDiff = Root::getSingleton().getNextFrameNumber() - mLastVisibleFrame;
 			if (frameDiff > 1 || frameDiff < 0) // < 0 if wrap only
 			{
 				mTimeSinceLastVisible += timeElapsed;
@@ -643,7 +643,7 @@ namespace Ogre {
         // Create new particles
         for( size_t i = oldSize; i < size; i++ )
 		{
-            mParticlePool[i] = new Particle();
+            mParticlePool[i] = OGRE_NEW Particle();
 		}
 
 		if (mIsRendererConfigured)
@@ -713,6 +713,16 @@ namespace Ogre {
             mRenderer->_updateRenderQueue(queue, mActiveParticles, mCullIndividual);
         }
     }
+	//---------------------------------------------------------------------
+	void ParticleSystem::visitRenderables(Renderable::Visitor* visitor, 
+		bool debugRenderables)
+	{
+		if (mRenderer)
+		{
+			mRenderer->visitRenderables(visitor, debugRenderables);
+		}
+	}
+	//---------------------------------------------------------------------
     void ParticleSystem::initParameters(void)
     {
         if (createParamDictionary("ParticleSystem"))
@@ -926,21 +936,24 @@ namespace Ogre {
 		MovableObject::_notifyCurrentCamera(cam);
 
 		// Record visible
-		mLastVisibleFrame = Root::getSingleton().getCurrentFrameNumber();
-		mTimeSinceLastVisible = 0.0f;
+		if (isVisible())
+		{			
+			mLastVisibleFrame = Root::getSingleton().getNextFrameNumber();
+			mTimeSinceLastVisible = 0.0f;
 
-        if (mSorted)
-		{
-			_sortParticles(cam);
+			if (mSorted)
+			{
+				_sortParticles(cam);
+			}
+
+			if (mRenderer)
+			{
+				if (!mIsRendererConfigured)
+					configureRenderer();
+
+				mRenderer->_notifyCurrentCamera(cam);
+			}
 		}
-
-		if (mRenderer)
-        {
-			if (!mIsRendererConfigured)
-				configureRenderer();
-
-            mRenderer->_notifyCurrentCamera(cam);
-        }
     }
     //-----------------------------------------------------------------------
     void ParticleSystem::_notifyAttached(Node* parent, bool isTagPoint)
@@ -955,11 +968,11 @@ namespace Ogre {
         {
             // Assume visible
             mTimeSinceLastVisible = 0;
-            mLastVisibleFrame = Root::getSingleton().getCurrentFrameNumber();
+            mLastVisibleFrame = Root::getSingleton().getNextFrameNumber();
 
             // Create time controller when attached
             ControllerManager& mgr = ControllerManager::getSingleton(); 
-            ControllerValueRealPtr updValue(new ParticleSystemUpdateValue(this));
+            ControllerValueRealPtr updValue(OGRE_NEW ParticleSystemUpdateValue(this));
             mTimeController = mgr.createFrameTimePassthroughController(updValue);
         }
         else if (!parent && mTimeController)
@@ -1118,9 +1131,7 @@ namespace Ogre {
     void ParticleSystem::setBounds(const AxisAlignedBox& aabb)
     {
         mAABB = aabb;
-        Real sqDist = std::max(mAABB.getMinimum().squaredLength(), 
-            mAABB.getMaximum().squaredLength());
-        mBoundingRadius = Math::Sqrt(sqDist);
+        mBoundingRadius = Math::boundingRadiusFromAABB(mAABB);
 
     }
     //-----------------------------------------------------------------------
@@ -1563,7 +1574,7 @@ namespace Ogre {
         std::vector<ParticleAffector*>::iterator i;
         for (i = mAffectors.begin(); i != mAffectors.end(); ++i)
         {
-            delete (*i);
+            OGRE_DELETE (*i);
         }
             
         mAffectors.clear();
@@ -1578,7 +1589,7 @@ namespace Ogre {
             if ((*i) == e)
             {
                 mAffectors.erase(i);
-                delete e;
+                OGRE_DELETE e;
                 break;
             }
         }
