@@ -49,8 +49,10 @@ template <class U> struct PixelBoxConverter
     static const int ID = U::ID;
     static void conversion(const PixelBox &src, const PixelBox &dst)
     {
-        typename U::SrcType *srcptr = static_cast<typename U::SrcType*>(src.data);
-        typename U::DstType *dstptr = static_cast<typename U::DstType*>(dst.data);
+        typename U::SrcType *srcptr = static_cast<typename U::SrcType*>(src.data)
+			+ (src.left + src.top * src.rowPitch + src.front * src.slicePitch);
+        typename U::DstType *dstptr = static_cast<typename U::DstType*>(dst.data)
+			+ (dst.left + dst.top * dst.rowPitch + dst.front * dst.slicePitch);
         const size_t srcSliceSkip = src.getSliceSkip();
         const size_t dstSliceSkip = dst.getSliceSkip();
         const size_t k = src.right - src.left;
