@@ -45,7 +45,7 @@ namespace Ogre {
     */
     struct PixelFormatDescription {
         /* Name of the format, as in the enum */
-        char *name;
+        const char *name;
         /* Number of bytes one element (colour value) takes. */
         unsigned char elemBytes;
         /* Pixel format flags, see enum PixelFormatFlags for the bit field
@@ -384,7 +384,7 @@ namespace Ogre {
         /* Bytes per element */
         8,
         /* Flags */
-        PFF_FLOAT,
+        PFF_FLOAT | PFF_HASALPHA,
         /* Component type and count */
         PCT_FLOAT16, 4,
         /* rbits, gbits, bbits, abits */
@@ -410,7 +410,7 @@ namespace Ogre {
         /* Bytes per element */
         16,
         /* Flags */
-        PFF_FLOAT,
+        PFF_FLOAT | PFF_HASALPHA,
         /* Component type and count */
         PCT_FLOAT32, 4,
         /* rbits, gbits, bbits, abits */
@@ -602,8 +602,10 @@ namespace Ogre {
 
 		const size_t elemSize = PixelUtil::getNumElemBytes(format);
 		// Calculate new data origin
-		PixelBox rval(def, format, ((uint8*)data)
-			+ ((def.left-left)*elemSize)
+		// Notice how we do not propagate left/top/front from the incoming box, since
+		// the returned pointer is already offset
+		PixelBox rval(def.getWidth(), def.getHeight(), def.getDepth(), format, 
+			((uint8*)data) + ((def.left-left)*elemSize)
 			+ ((def.top-top)*rowPitch*elemSize)
 			+ ((def.front-front)*slicePitch*elemSize)
 		);
@@ -1202,10 +1204,12 @@ namespace Ogre {
                 return;
             }
 
-            uint8 *srcptr = static_cast<uint8*>(src.data);
-            uint8 *dstptr = static_cast<uint8*>(dst.data);
             const size_t srcPixelSize = PixelUtil::getNumElemBytes(src.format);
             const size_t dstPixelSize = PixelUtil::getNumElemBytes(dst.format);
+            uint8 *srcptr = static_cast<uint8*>(src.data)
+                + (src.left + src.top * src.rowPitch + src.front * src.slicePitch) * srcPixelSize;
+            uint8 *dstptr = static_cast<uint8*>(dst.data)
+				+ (dst.left + dst.top * dst.rowPitch + dst.front * dst.slicePitch) * dstPixelSize;
 
             // Calculate pitches+skips in bytes
             const size_t srcRowPitchBytes = src.rowPitch*srcPixelSize;
@@ -1264,10 +1268,12 @@ namespace Ogre {
         }
 #endif
 
-        uint8 *srcptr = static_cast<uint8*>(src.data);
-        uint8 *dstptr = static_cast<uint8*>(dst.data);
         const size_t srcPixelSize = PixelUtil::getNumElemBytes(src.format);
         const size_t dstPixelSize = PixelUtil::getNumElemBytes(dst.format);
+        uint8 *srcptr = static_cast<uint8*>(src.data)
+            + (src.left + src.top * src.rowPitch + src.front * src.slicePitch) * srcPixelSize;
+        uint8 *dstptr = static_cast<uint8*>(dst.data)
+            + (dst.left + dst.top * dst.rowPitch + dst.front * dst.slicePitch) * dstPixelSize;
 
         // Calculate pitches+skips in bytes
         const size_t srcRowSkipBytes = src.getRowSkip()*srcPixelSize;
