@@ -48,14 +48,12 @@ Torus Knot Software Ltd.
 
 namespace Ogre {
 
-    const PlaneList Renderable::msDummyPlaneList; // FIX ME: temporary
     static const TexturePtr sNullTexPtr;
 
     //-----------------------------------------------------------------------
     RenderSystem::RenderSystem()
         : mActiveRenderTarget(0)
         , mTextureManager(0)
-        , mCapabilities(0)
         , mActiveViewport(0)
         // This means CULL clockwise vertices, i.e. front of poly is counter-clockwise
         // This makes it the same as OpenGL and other right-handed systems
@@ -65,19 +63,27 @@ namespace Ogre {
         , mInvertVertexWinding(false)
         , mDisabledTexUnitsFrom(0)
         , mCurrentPassIterationCount(0)
+		, mDerivedDepthBias(false)
         , mVertexProgramBound(false)
+		, mGeometryProgramBound(false)
         , mFragmentProgramBound(false)
+		, mClipPlanesDirty(true)
+		, mRealCapabilities(0)
+		, mCurrentCapabilities(0)
+		, mUseCustomCapabilities(false)
+		, mTexProjRelative(false)
+		, mTexProjRelativeOrigin(Vector3::ZERO)
     {
-        // instanciate RenderSystemCapabilities
-        mCapabilities = new RenderSystemCapabilities();
     }
 
     //-----------------------------------------------------------------------
     RenderSystem::~RenderSystem()
     {
         shutdown();
-		delete mCapabilities;
-		mCapabilities = 0;
+		OGRE_DELETE mRealCapabilities;
+		mRealCapabilities = 0;
+		// Current capabilities managed externally
+		mCurrentCapabilities = 0;
     }
     //-----------------------------------------------------------------------
     void RenderSystem::_initRenderTargets(void)
@@ -94,7 +100,7 @@ namespace Ogre {
 
     }
     //-----------------------------------------------------------------------
-    void RenderSystem::_updateAllRenderTargets(void)
+    void RenderSystem::_updateAllRenderTargets(bool swapBuffers)
     {
         // Update all in order of priority
         // This ensures render-to-texture targets get updated before render windows
@@ -103,11 +109,24 @@ namespace Ogre {
 		for( itarg = mPrioritisedRenderTargets.begin(); itarg != itargend; ++itarg )
 		{
 			if( itarg->second->isActive() && itarg->second->isAutoUpdated())
-				itarg->second->update();
+				itarg->second->update(swapBuffers);
 		}
     }
     //-----------------------------------------------------------------------
-    RenderWindow* RenderSystem::initialise(bool autoCreateWindow, const String& windowTitle)
+    void RenderSystem::_swapAllRenderTargetBuffers(bool waitForVSync)
+    {
+        // Update all in order of priority
+        // This ensures render-to-texture targets get updated before render windows
+		RenderTargetPriorityMap::iterator itarg, itargend;
+		itargend = mPrioritisedRenderTargets.end();
+		for( itarg = mPrioritisedRenderTargets.begin(); itarg != itargend; ++itarg )
+		{
+			if( itarg->second->isActive() && itarg->second->isAutoUpdated())
+				itarg->second->swapBuffers(waitForVSync);
+		}
+    }
+    //-----------------------------------------------------------------------
+    RenderWindow* RenderSystem::_initialise(bool autoCreateWindow, const String& windowTitle)
     {
         // Have I been registered by call to Root::setRenderSystem?
 		/** Don't do this anymore, just allow via Root
@@ -123,25 +142,17 @@ namespace Ogre {
         //   their own initialise() implementations.
         
         mVertexProgramBound = false;
+		mGeometryProgramBound = false;
         mFragmentProgramBound = false;
 
         return 0;
     }
-	//---------------------------------------------------------------------
-	RenderTexture * RenderSystem::createRenderTexture( const String & name, 
-		unsigned int width, unsigned int height,
-		TextureType texType, PixelFormat internalFormat, const NameValuePairList *miscParams )
-	{
-		/// Create a new 2D texture, and return surface to render to
-        TexturePtr mTexture = TextureManager::getSingleton().createManual( name, 
-			ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME, texType, 
-			width, height, 0, internalFormat, TU_RENDERTARGET );
-            
-        // Ensure texture loaded and internal resources created
-        mTexture->load();
 
-        return mTexture->getBuffer()->getRenderTarget();
-	}
+		void RenderSystem::useCustomRenderSystemCapabilities(RenderSystemCapabilities* capabilities)
+		{
+				mCurrentCapabilities = capabilities;
+				mUseCustomCapabilities = true;
+		}
     //---------------------------------------------------------------------------------------------
     void RenderSystem::destroyRenderWindow(const String& name)
     {
@@ -156,7 +167,7 @@ namespace Ogre {
     void RenderSystem::destroyRenderTarget(const String& name)
     {
         RenderTarget* rt = detachRenderTarget(name);
-        delete rt;
+        OGRE_DELETE rt;
     }
     //---------------------------------------------------------------------------------------------
     void RenderSystem::attachRenderTarget( RenderTarget &target )
@@ -224,8 +235,8 @@ namespace Ogre {
 
         const TexturePtr& tex = tl._getTexturePtr();
 		// Vertex texture binding?
-		if (mCapabilities->hasCapability(RSC_VERTEX_TEXTURE_FETCH) && 
-			!mCapabilities->getVertexTextureUnitsShared())
+		if (mCurrentCapabilities->hasCapability(RSC_VERTEX_TEXTURE_FETCH) &&
+			!mCurrentCapabilities->getVertexTextureUnitsShared())
 		{
 			if (tl.getBindingType() == TextureUnitState::BT_VERTEX)
 			{
@@ -358,7 +369,7 @@ namespace Ogre {
     //---------------------------------------------------------------------
     void RenderSystem::_disableTextureUnitsFrom(size_t texUnit)
     {
-        size_t disableTo = mCapabilities->getNumTextureUnits();
+        size_t disableTo = OGRE_MAX_TEXTURE_LAYERS;
         if (disableTo > mDisabledTexUnitsFrom)
             disableTo = mDisabledTexUnitsFrom;
         mDisabledTexUnitsFrom = texUnit;
@@ -406,7 +417,7 @@ namespace Ogre {
 		for (HardwareOcclusionQueryList::iterator i = mHwOcclusionQueries.begin();
 			i != mHwOcclusionQueries.end(); ++i)
 		{
-			delete *i;
+			OGRE_DELETE *i;
 		}
 		mHwOcclusionQueries.clear();
 
@@ -418,9 +429,9 @@ namespace Ogre {
 			if (!primary && it->second->isPrimary())
 				primary = it->second;
 			else
-				delete it->second;
+				OGRE_DELETE it->second;
 		}
-		delete primary;
+		OGRE_DELETE primary;
 		mRenderTargets.clear();
 
 		mPrioritisedRenderTargets.clear();
@@ -475,6 +486,7 @@ namespace Ogre {
         // account for a pass having multiple iterations
         if (mCurrentPassIterationCount > 1)
             val *= mCurrentPassIterationCount;
+		mCurrentPassIterationNum = 0;
 
         switch(op.operationType)
         {
@@ -493,17 +505,49 @@ namespace Ogre {
 
         mVertexCount += op.vertexData->vertexCount;
         mBatchCount += mCurrentPassIterationCount;
+
+		// sort out clip planes
+		// have to do it here in case of matrix issues
+		if (mClipPlanesDirty)
+		{
+			setClipPlanesImpl(mClipPlanes);
+			mClipPlanesDirty = false;
+		}
     }
     //-----------------------------------------------------------------------
     void RenderSystem::setInvertVertexWinding(bool invert)
     {
         mInvertVertexWinding = invert;
     }
-    //-----------------------------------------------------------------------
-    void RenderSystem::setClipPlane (ushort index, const Plane &p)
-    {
-        setClipPlane (index, p.normal.x, p.normal.y, p.normal.z, p.d);
-    }
+	//---------------------------------------------------------------------
+	void RenderSystem::addClipPlane (const Plane &p)
+	{
+		mClipPlanes.push_back(p);
+		mClipPlanesDirty = true;
+	}
+	//---------------------------------------------------------------------
+	void RenderSystem::addClipPlane (Real A, Real B, Real C, Real D)
+	{
+		addClipPlane(Plane(A, B, C, D));
+	}
+	//---------------------------------------------------------------------
+	void RenderSystem::setClipPlanes(const PlaneList& clipPlanes)
+	{
+		if (clipPlanes != mClipPlanes)
+		{
+			mClipPlanes = clipPlanes;
+			mClipPlanesDirty = true;
+		}
+	}
+	//---------------------------------------------------------------------
+	void RenderSystem::resetClipPlanes()
+	{
+		if (!mClipPlanes.empty())
+		{
+			mClipPlanes.clear();
+			mClipPlanesDirty = true;
+		}
+	}
     //-----------------------------------------------------------------------
     void RenderSystem::_notifyCameraRemoved(const Camera* cam)
     {
@@ -523,11 +567,17 @@ namespace Ogre {
             return false;
 
         --mCurrentPassIterationCount;
+		++mCurrentPassIterationNum;
         if (!mActiveVertexGpuProgramParameters.isNull())
         {
             mActiveVertexGpuProgramParameters->incPassIterationNumber();
             bindGpuProgramPassIterationParameters(GPT_VERTEX_PROGRAM);
         }
+        if (!mActiveGeometryGpuProgramParameters.isNull())
+        {
+            mActiveGeometryGpuProgramParameters->incPassIterationNumber();
+            bindGpuProgramPassIterationParameters(GPT_GEOMETRY_PROGRAM);
+        }
         if (!mActiveFragmentGpuProgramParameters.isNull())
         {
             mActiveFragmentGpuProgramParameters->incPassIterationNumber();
@@ -563,7 +613,7 @@ namespace Ogre {
 		if (i != mHwOcclusionQueries.end())
 		{
 			mHwOcclusionQueries.erase(i);
-			delete hq;
+			OGRE_DELETE hq;
 		}
 	}
 	//-----------------------------------------------------------------------
@@ -572,8 +622,15 @@ namespace Ogre {
 	    switch(prg->getType())
 	    {
         case GPT_VERTEX_PROGRAM:
+			// mark clip planes dirty if changed (programmable can change space)
+			if (!mVertexProgramBound && !mClipPlanes.empty())
+				mClipPlanesDirty = true;
+
             mVertexProgramBound = true;
 	        break;
+        case GPT_GEOMETRY_PROGRAM:
+			mGeometryProgramBound = true;
+			break;
         case GPT_FRAGMENT_PROGRAM:
             mFragmentProgramBound = true;
 	        break;
@@ -585,8 +642,14 @@ namespace Ogre {
 	    switch(gptype)
 	    {
         case GPT_VERTEX_PROGRAM:
+			// mark clip planes dirty if changed (programmable can change space)
+			if (mVertexProgramBound && !mClipPlanes.empty())
+				mClipPlanesDirty = true;
             mVertexProgramBound = false;
 	        break;
+        case GPT_GEOMETRY_PROGRAM:
+			mGeometryProgramBound = false;
+			break;
         case GPT_FRAGMENT_PROGRAM:
             mFragmentProgramBound = false;
 	        break;
@@ -599,12 +662,21 @@ namespace Ogre {
 	    {
         case GPT_VERTEX_PROGRAM:
             return mVertexProgramBound;
+        case GPT_GEOMETRY_PROGRAM:
+            return mGeometryProgramBound;
         case GPT_FRAGMENT_PROGRAM:
             return mFragmentProgramBound;
 	    }
         // Make compiler happy
         return false;
 	}
+	//---------------------------------------------------------------------
+	void RenderSystem::_setTextureProjectionRelativeTo(bool enabled, const Vector3& pos)
+	{
+		mTexProjRelative = enabled;
+		mTexProjRelativeOrigin = pos;
+
+	}
 
 }
 
