@@ -59,7 +59,7 @@ namespace Ogre
 	{
 		setTrailLength(100);
 		setNumberOfChains(numberOfChains);
-		mTimeControllerValue = ControllerValueRealPtr(new TimeControllerValue(this));
+		mTimeControllerValue = ControllerValueRealPtr(OGRE_NEW TimeControllerValue(this));
 
 		// use V as varying texture coord, so we can use 1D textures to 'smear'
 		setTextureCoordDirection(TCD_V);
@@ -197,10 +197,10 @@ namespace Ogre
 		}
 		else if (oldChains < numChains)
 		{
-			// add new chains, in reverse order to preserve previous ordering (pop_back)
+			// add new chains, at front to preserve previous ordering (pop_back)
 			int count = static_cast<int>(numChains - oldChains);
-			for (size_t i = numChains - 1; count > 0; --i, --count)
-				mFreeChains.push_back(i);
+			for (size_t i = oldChains; i < numChains; ++i)
+				mFreeChains.insert(mFreeChains.begin(), i);
 		}
         resetAllTrails();
 	}
@@ -541,13 +541,13 @@ namespace Ogre
 
 		}
 
-		return new RibbonTrail(name, maxElements, numberOfChains, useTex, useCol);
+		return OGRE_NEW RibbonTrail(name, maxElements, numberOfChains, useTex, useCol);
 
 	}
 	//-----------------------------------------------------------------------
 	void RibbonTrailFactory::destroyInstance( MovableObject* obj)
 	{
-		delete obj;
+		OGRE_DELETE obj;
 	}
 
 
