@@ -74,9 +74,8 @@ namespace Ogre {
     void Skeleton::loadImpl(void)
     {
         SkeletonSerializer serializer;
-		StringUtil::StrStreamType msg;
-		msg << "Skeleton: Loading " << mName;
-        LogManager::getSingleton().logMessage(msg.str());
+		LogManager::getSingleton().stream()
+			<< "Skeleton: Loading " << mName;
 
         DataStreamPtr stream = 
             ResourceGroupManager::getSingleton().openResource(
@@ -102,7 +101,7 @@ namespace Ogre {
         BoneList::iterator i;
         for (i = mBoneList.begin(); i != mBoneList.end(); ++i)
         {
-            delete *i;
+            OGRE_DELETE *i;
         }
         mBoneList.clear();
         mBoneListByName.clear();
@@ -114,7 +113,7 @@ namespace Ogre {
         AnimationList::iterator ai;
         for (ai = mAnimationsList.begin(); ai != mAnimationsList.end(); ++ai)
         {
-            delete ai->second;
+            OGRE_DELETE ai->second;
         }
         mAnimationsList.clear();
 
@@ -148,7 +147,7 @@ namespace Ogre {
                 "A bone with the handle " + StringConverter::toString(handle) + " already exists",
                 "Skeleton::createBone" );
         }
-        Bone* ret = new Bone(handle, this);
+        Bone* ret = OGRE_NEW Bone(handle, this);
         assert(mBoneListByName.find(ret->getName()) == mBoneListByName.end());
         if (mBoneList.size() <= handle)
         {
@@ -183,7 +182,7 @@ namespace Ogre {
                 "A bone with the name " + name + " already exists",
                 "Skeleton::createBone" );
         }
-        Bone* ret = new Bone(name, handle, this);
+        Bone* ret = OGRE_NEW Bone(name, handle, this);
         if (mBoneList.size() <= handle)
         {
             mBoneList.resize(handle+1);
@@ -253,16 +252,16 @@ namespace Ogre {
             // tolerate state entries for animations we're not aware of
             if (anim)
             {
-                if (linked)
-                {
-                    anim->apply(this, animState->getTimePosition(), 
-						animState->getWeight() * weightFactor, linked->scale);
-                }
-                else
-                {
-                    anim->apply(this, animState->getTimePosition(), 
-						animState->getWeight() * weightFactor);
-                }
+              if(animState->hasBlendMask())
+              {
+                anim->apply(this, animState->getTimePosition(), animState->getWeight() * weightFactor,
+                  animState->getBlendMask(), linked ? linked->scale : 1.0f);
+              }
+              else
+              {
+                anim->apply(this, animState->getTimePosition(), 
+                  animState->getWeight() * weightFactor, linked ? linked->scale : 1.0f);
+              }
             }
         }
 
@@ -303,7 +302,7 @@ namespace Ogre {
                 "Skeleton::createAnimation");
         }
 
-        Animation* ret = new Animation(name, length);
+        Animation* ret = OGRE_NEW Animation(name, length);
 
         // Add to list
         mAnimationsList[name] = ret;
@@ -375,7 +374,7 @@ namespace Ogre {
             "Skeleton::getAnimation");
         }
 
-        delete i->second;
+        OGRE_DELETE i->second;
 
         mAnimationsList.erase(i);
 
@@ -521,6 +520,11 @@ namespace Ogre {
         return i->second;
 
     }
+	//---------------------------------------------------------------------
+	bool Skeleton::hasBone(const String& name) const 
+	{	
+		return mBoneListByName.find(name) != mBoneListByName.end();
+	}
     //---------------------------------------------------------------------
     void Skeleton::deriveRootBone(void) const
     {
