@@ -87,11 +87,11 @@ namespace Ogre {
         }
         // Now split args into that god-awful char** that Cg insists on
         freeCgArgs();
-        mCgArguments = new char*[args.size() + 1];
+        mCgArguments = OGRE_ALLOC_T(char*, args.size() + 1, MEMCATEGORY_RESOURCE);
         int index = 0;
         for (i = args.begin(); i != args.end(); ++i, ++index)
         {
-            mCgArguments[index] = new char[i->length() + 1];
+            mCgArguments[index] = OGRE_ALLOC_T(char, i->length() + 1, MEMCATEGORY_RESOURCE);
             strcpy(mCgArguments[index], i->c_str());
         }
         // Null terminate list
@@ -108,10 +108,11 @@ namespace Ogre {
             char* current = mCgArguments[index];
             while (current)
             {
-                delete [] current;
+                OGRE_FREE(current, MEMCATEGORY_RESOURCE);
+				mCgArguments[index] = 0;
                 current = mCgArguments[++index];
             }
-            delete [] mCgArguments;
+            OGRE_FREE(mCgArguments, MEMCATEGORY_RESOURCE);
             mCgArguments = 0;
         }
     }
@@ -128,7 +129,9 @@ namespace Ogre {
 			return;
 		}
         buildArgs();
-        mCgProgram = cgCreateProgram(mCgContext, CG_SOURCE, mSource.c_str(), 
+		// deal with includes
+		String sourceToUse = preprocess(mSource);
+        mCgProgram = cgCreateProgram(mCgContext, CG_SOURCE, sourceToUse.c_str(), 
             mSelectedCgProfile, mEntryPoint.c_str(), const_cast<const char**>(mCgArguments));
 
         // Test
@@ -143,7 +146,7 @@ namespace Ogre {
     void CgProgram::createLowLevelImpl(void)
     {
 		// ignore any previous error
-		if (mSelectedCgProfile != CG_PROFILE_UNKNOWN)
+		if (mSelectedCgProfile != CG_PROFILE_UNKNOWN && !mCompileError)
 		{
 
 			// Create a low-level program, give it the same name as us
@@ -154,6 +157,9 @@ namespace Ogre {
 					cgGetProgramString(mCgProgram, CG_COMPILED_PROGRAM),
 					mType, 
 					mSelectedProfile);
+
+			// Shader params need to be forwarded to low level implementation
+			mAssemblerProgram->setAdjacencyInfoRequired(isAdjacencyInfoRequired());
 		}
     }
     //-----------------------------------------------------------------------
@@ -262,6 +268,7 @@ namespace Ogre {
 							"Problem parsing the following Cg Uniform: '"
 							+ paramName + "' in file " + mName);
 						// next uniform
+						parameter = cgGetNextParameter(parameter);
 						continue;
 					}
 					if (isRegisterCombiner)
@@ -282,6 +289,7 @@ namespace Ogre {
 					}
 
 
+					def.logicalIndex = logicalIndex;
 					mConstantDefs.map.insert(GpuConstantDefinitionMap::value_type(paramName, def));
 
 					// Record logical / physical mapping
@@ -493,7 +501,121 @@ namespace Ogre {
             mProfiles.push_back(*i);
         }
     }
+	//-----------------------------------------------------------------------
+	String CgProgram::preprocess(const String& inSource)
+	{
+		String outSource;
+		// output will be at least this big
+		outSource.reserve(inSource.length());
 
+		size_t startMarker = 0;
+		size_t i = inSource.find("#include");
+		while (i != String::npos)
+		{
+			size_t includePos = i;
+			size_t afterIncludePos = includePos + 8;
+			size_t newLineBefore = inSource.rfind("\n", includePos);
+
+			// check we're not in a comment
+			size_t lineCommentIt = inSource.rfind("//", includePos);
+			if (lineCommentIt != String::npos)
+			{
+				if (newLineBefore == String::npos || lineCommentIt > newLineBefore)
+				{
+					// commented
+					i = inSource.find("#include", afterIncludePos);
+					continue;
+				}
+
+			}
+			size_t blockCommentIt = inSource.rfind("/*", includePos);
+			if (blockCommentIt != String::npos)
+			{
+				size_t closeCommentIt = inSource.rfind("*/", includePos);
+				if (closeCommentIt == String::npos || closeCommentIt < blockCommentIt)
+				{
+					// commented
+					i = inSource.find("#include", afterIncludePos);
+					continue;
+				}
+
+			}
+
+			// find following newline (or EOF)
+			size_t newLineAfter = inSource.find("\n", afterIncludePos);
+			// find include file string container
+			String endDelimeter = "\"";
+			size_t startIt = inSource.find("\"", afterIncludePos);
+			if (startIt == String::npos || startIt > newLineAfter)
+			{
+				// try <>
+				startIt = inSource.find("<", afterIncludePos);
+				if (startIt == String::npos || startIt > newLineAfter)
+				{
+					OGRE_EXCEPT(Exception::ERR_INTERNAL_ERROR,
+						"Badly formed #include directive (expected \" or <) in file "
+						+ mFilename + ": " + inSource.substr(includePos, newLineAfter-includePos),
+						"CgProgram::preprocessor");
+				}
+				else
+				{
+					endDelimeter = ">";
+				}
+			}
+			size_t endIt = inSource.find(endDelimeter, startIt+1);
+			if (endIt == String::npos || endIt <= startIt)
+			{
+				OGRE_EXCEPT(Exception::ERR_INTERNAL_ERROR,
+					"Badly formed #include directive (expected " + endDelimeter + ") in file "
+					+ mFilename + ": " + inSource.substr(includePos, newLineAfter-includePos),
+					"CgProgram::preprocessor");
+			}
+
+			// extract filename
+			String filename(inSource.substr(startIt+1, endIt-startIt-1));
+
+			// open included file
+			DataStreamPtr resource = ResourceGroupManager::getSingleton().
+				openResource(filename, mGroup, true, this);
+
+			// replace entire include directive line
+			// copy up to just before include
+			if (newLineBefore != String::npos && newLineBefore >= startMarker)
+				outSource.append(inSource.substr(startMarker, newLineBefore-startMarker+1));
+
+			size_t lineCount = 0;
+			size_t lineCountPos = 0;
+			
+			// Count the line number of #include statement
+			lineCountPos = outSource.find('\n');
+			while(lineCountPos != String::npos)
+			{
+				lineCountPos = outSource.find('\n', lineCountPos+1);
+				lineCount++;
+			}
+
+			// Add #line to the start of the included file to correct the line count
+			outSource.append("#line 1 \"" + filename + "\"\n");
+
+			outSource.append(resource->getAsString());
+
+			// Add #line to the end of the included file to correct the line count
+			outSource.append("\n#line " + Ogre::StringConverter::toString(lineCount) + 
+				"\"" + this->getSourceFile() + "\"\n");
+
+			startMarker = newLineAfter;
+
+			if (startMarker != String::npos)
+				i = inSource.find("#include", startMarker);
+			else
+				i = String::npos;
+
+		}
+		// copy any remaining characters
+		outSource.append(inSource.substr(startMarker));
+
+		return outSource;
+	}
     //-----------------------------------------------------------------------
     const String& CgProgram::getLanguage(void) const
     {
