@@ -316,19 +316,11 @@ const String& OctreeSceneManager::getTypeName(void) const
 
 void OctreeSceneManager::init( AxisAlignedBox &box, int depth )
 {
-    delete mSceneRoot; //get rid of old root.
-
-    // -- Changes by Steve
-    // Don't do it this way, it will add it to the mSceneNodes which we don't want
-    //mSceneRoot = createSceneNode( "SceneRoot" );
-    mSceneRoot = new OctreeNode( this, "SceneRoot" );
-	mSceneRoot->_notifyRootNode();
-    // -- End changes by Steve
 
     if ( mOctree != 0 )
-        delete mOctree;
+        OGRE_DELETE mOctree;
 
-    mOctree = new Octree( 0 );
+    mOctree = OGRE_NEW Octree( 0 );
 
     mMaxDepth = depth;
     mBox = box;
@@ -362,24 +354,26 @@ void OctreeSceneManager::init( AxisAlignedBox &box, int depth )
 
 OctreeSceneManager::~OctreeSceneManager()
 {
-    // -- Changed by Steve
-    // Don't do this here, SceneManager will do it
-    /*
-    if( mSceneRoot )
-    delete mSceneRoot;
-    */ 
-    // --End Changes by Steve
 
     if ( mOctree )
 	{
-        delete mOctree;
+        OGRE_DELETE mOctree;
 		mOctree = 0;
 	}
 }
 
 Camera * OctreeSceneManager::createCamera( const String &name )
 {
-    Camera * c = new OctreeCamera( name, this );
+	// Check name not used
+	if (mCameras.find(name) != mCameras.end())
+	{
+		OGRE_EXCEPT(
+			Exception::ERR_DUPLICATE_ITEM,
+			"A camera with the name " + name + " already exists",
+			"OctreeSceneManager::createCamera" );
+	}
+
+	Camera * c = OGRE_NEW OctreeCamera( name, this );
     mCameras.insert( CameraList::value_type( name, c ) );
 
 	// create visible bounds aab map entry
@@ -485,7 +479,7 @@ void OctreeSceneManager::_addOctreeNode( OctreeNode * n, Octree *octant, int dep
 
         if ( octant -> mChildren[ x ][ y ][ z ] == 0 )
         {
-            octant -> mChildren[ x ][ y ][ z ] = new Octree( octant );
+            octant -> mChildren[ x ][ y ][ z ] = OGRE_NEW Octree( octant );
             const Vector3& octantMin = octant -> mBox.getMinimum();
             const Vector3& octantMax = octant -> mBox.getMaximum();
             Vector3 min, max;
@@ -541,26 +535,14 @@ void OctreeSceneManager::_addOctreeNode( OctreeNode * n, Octree *octant, int dep
 }
 
 
-SceneNode * OctreeSceneManager::createSceneNode( void )
+SceneNode * OctreeSceneManager::createSceneNodeImpl( void )
 {
-    OctreeNode * on = new OctreeNode( this );
-    mSceneNodes[ on->getName() ] = on;
-    return on;
+    return OGRE_NEW OctreeNode( this );
 }
 
-SceneNode * OctreeSceneManager::createSceneNode( const String &name )
+SceneNode * OctreeSceneManager::createSceneNodeImpl( const String &name )
 {
-    // Check name not used
-    if (mSceneNodes.find(name) != mSceneNodes.end())
-    {
-        OGRE_EXCEPT(
-            Exception::ERR_DUPLICATE_ITEM,
-            "A scene node with the name " + name + " already exists",
-            "OctreeSceneManager::createSceneNode" );
-    }
-    OctreeNode * on = new OctreeNode( this, name );
-    mSceneNodes[ on->getName() ] = on;
-    return on;
+    return OGRE_NEW OctreeNode( this, name );
 }
 
 void OctreeSceneManager::_updateSceneGraph( Camera * cam )
@@ -1031,9 +1013,9 @@ void OctreeSceneManager::resize( const AxisAlignedBox &box )
 
     _findNodes( mOctree->mBox, nodes, 0, true, mOctree );
 
-    delete mOctree;
+    OGRE_DELETE mOctree;
 
-    mOctree = new Octree( 0 );
+    mOctree = OGRE_NEW Octree( 0 );
     mOctree->mBox = box;
 
 	const Vector3 min = box.getMinimum();
@@ -1119,7 +1101,7 @@ void OctreeSceneManager::clearScene(void)
 AxisAlignedBoxSceneQuery*
 OctreeSceneManager::createAABBQuery(const AxisAlignedBox& box, unsigned long mask)
 {
-    OctreeAxisAlignedBoxSceneQuery* q = new OctreeAxisAlignedBoxSceneQuery(this);
+    OctreeAxisAlignedBoxSceneQuery* q = OGRE_NEW OctreeAxisAlignedBoxSceneQuery(this);
     q->setBox(box);
     q->setQueryMask(mask);
     return q;
@@ -1128,7 +1110,7 @@ OctreeSceneManager::createAABBQuery(const AxisAlignedBox& box, unsigned long mas
 SphereSceneQuery*
 OctreeSceneManager::createSphereQuery(const Sphere& sphere, unsigned long mask)
 {
-    OctreeSphereSceneQuery* q = new OctreeSphereSceneQuery(this);
+    OctreeSphereSceneQuery* q = OGRE_NEW OctreeSphereSceneQuery(this);
     q->setSphere(sphere);
     q->setQueryMask(mask);
     return q;
@@ -1138,7 +1120,7 @@ PlaneBoundedVolumeListSceneQuery*
 OctreeSceneManager::createPlaneBoundedVolumeQuery(const PlaneBoundedVolumeList& volumes,
         unsigned long mask)
 {
-    OctreePlaneBoundedVolumeListSceneQuery* q = new OctreePlaneBoundedVolumeListSceneQuery(this);
+    OctreePlaneBoundedVolumeListSceneQuery* q = OGRE_NEW OctreePlaneBoundedVolumeListSceneQuery(this);
     q->setVolumes(volumes);
     q->setQueryMask(mask);
     return q;
@@ -1148,7 +1130,7 @@ OctreeSceneManager::createPlaneBoundedVolumeQuery(const PlaneBoundedVolumeList&
 RaySceneQuery*
 OctreeSceneManager::createRayQuery(const Ray& ray, unsigned long mask)
 {
-    OctreeRaySceneQuery* q = new OctreeRaySceneQuery(this);
+    OctreeRaySceneQuery* q = OGRE_NEW OctreeRaySceneQuery(this);
     q->setRay(ray);
     q->setQueryMask(mask);
     return q;
@@ -1160,8 +1142,8 @@ OctreeSceneManager::createIntersectionQuery(unsigned long mask)
 
     // Octree implementation performs WORSE for < 500 objects
     // TODO: optimise it so it's better in all cases
-    //OctreeIntersectionSceneQuery* q = new OctreeIntersectionSceneQuery(this);
-    DefaultIntersectionSceneQuery* q = new DefaultIntersectionSceneQuery(this);
+    //OctreeIntersectionSceneQuery* q = OGRE_NEW OctreeIntersectionSceneQuery(this);
+    DefaultIntersectionSceneQuery* q = OGRE_NEW DefaultIntersectionSceneQuery(this);
     q->setQueryMask(mask);
     return q;
 }
@@ -1179,12 +1161,12 @@ void OctreeSceneManagerFactory::initMetaData(void) const
 SceneManager* OctreeSceneManagerFactory::createInstance(
 	const String& instanceName)
 {
-	return new OctreeSceneManager(instanceName);
+	return OGRE_NEW OctreeSceneManager(instanceName);
 }
 //-----------------------------------------------------------------------
 void OctreeSceneManagerFactory::destroyInstance(SceneManager* instance)
 {
-	delete instance;
+	OGRE_DELETE instance;
 }
 
 
