@@ -63,7 +63,7 @@ namespace Ogre
 
     //-----------------------------------------------------------------------
     TerrainRenderable::TerrainRenderable(const String& name, TerrainSceneManager* tsm)
-        : Renderable(), MovableObject(name), mSceneManager(tsm), mTerrain(0), mDeltaBuffers(0), mPositionBuffer(0)
+        : Renderable(), MovableObject(name), mSceneManager(tsm), mTerrain(0), mPositionBuffer(0)
     {
         mForcedRenderLevel = -1;
         mLastNextLevel = -1;
@@ -99,16 +99,13 @@ namespace Ogre
     void TerrainRenderable::deleteGeometry()
     {
         if(mTerrain)
-            delete mTerrain;
+            OGRE_DELETE mTerrain;
 
         if (mPositionBuffer)
-            delete [] mPositionBuffer;
-
-        if (mDeltaBuffers)
-            delete [] mDeltaBuffers;
+            OGRE_FREE(mPositionBuffer, MEMCATEGORY_GEOMETRY);
 
         if ( mMinLevelDistSqr != 0 )
-            delete [] mMinLevelDistSqr;
+            OGRE_FREE(mMinLevelDistSqr, MEMCATEGORY_GEOMETRY);
     }
     //-----------------------------------------------------------------------
     void TerrainRenderable::initialise(int startx, int startz,  
@@ -131,7 +128,7 @@ namespace Ogre
         //calculate min and max heights;
         Real min = 256000, max = 0;
 
-        mTerrain = new VertexData;
+        mTerrain = OGRE_NEW VertexData;
         mTerrain->vertexStart = 0;
         mTerrain->vertexCount = mOptions->tileSize * mOptions->tileSize;
 
@@ -165,7 +162,7 @@ namespace Ogre
             mTerrain->vertexCount, 
             HardwareBuffer::HBU_STATIC_WRITE_ONLY);
         // Create system memory copy with just positions in it, for use in simple reads
-        mPositionBuffer = new float[mTerrain->vertexCount * 3];
+        mPositionBuffer = OGRE_ALLOC_T(float, mTerrain->vertexCount * 3, MEMCATEGORY_GEOMETRY);
 
         bind->setBinding(MAIN_BINDING, mMainBuffer);
 
@@ -181,7 +178,7 @@ namespace Ogre
 
         mRenderLevel = 1;
 
-        mMinLevelDistSqr = new Real[ mOptions->maxGeoMipMapLevel ];
+        mMinLevelDistSqr = OGRE_ALLOC_T(Real, mOptions->maxGeoMipMapLevel, MEMCATEGORY_GEOMETRY);
 
         int endx = startx + mOptions->tileSize;
 
@@ -252,7 +249,7 @@ namespace Ogre
         if (mOptions->lodMorph)
         {
             // Create delta buffer for all except the lowest mip
-            mDeltaBuffers = new HardwareVertexBufferSharedPtr[mOptions->maxGeoMipMapLevel - 1];
+            mDeltaBuffers.resize(mOptions->maxGeoMipMapLevel - 1);
         }
 
         Real C = _calculateCFactor();
@@ -420,6 +417,12 @@ namespace Ogre
 
         queue->addRenderable(this, mRenderQueueID);
     }
+	//---------------------------------------------------------------------
+	void TerrainRenderable::visitRenderables(Renderable::Visitor* visitor, 
+		bool debugRenderables)
+	{
+		visitor->visit(this, 0, false);
+	}
     //-----------------------------------------------------------------------
     void TerrainRenderable::getRenderOperation( RenderOperation& op )
     {
@@ -441,16 +444,6 @@ namespace Ogre
         *xform = mParentNode->_getFullTransform();
     }
     //-----------------------------------------------------------------------
-    const Quaternion& TerrainRenderable::getWorldOrientation(void) const
-    {
-        return mParentNode->_getDerivedOrientation();
-    }
-    //-----------------------------------------------------------------------
-    const Vector3& TerrainRenderable::getWorldPosition(void) const
-    {
-        return mParentNode->_getDerivedPosition();
-    }
-    //-----------------------------------------------------------------------
     bool TerrainRenderable::_checkSize( int n )
     {
         for ( int i = 0; i < 10; i++ )
@@ -1027,7 +1020,7 @@ namespace Ogre
         int new_length = numTrisAcross * ((mOptions->tileSize-1) / step) + 2;
         //this is the maximum for a level.  It wastes a little, but shouldn't be a problem.
 
-        IndexData* indexData = new IndexData;
+        IndexData* indexData = OGRE_NEW IndexData;
         indexData->indexBuffer = 
             HardwareBufferManager::getSingleton().createIndexBuffer(
             HardwareIndexBuffer::IT_16BIT,
@@ -1219,7 +1212,7 @@ namespace Ogre
         int new_length = ( mOptions->tileSize / step ) * ( mOptions->tileSize / step ) * 2 * 2 * 2 ;
         //this is the maximum for a level.  It wastes a little, but shouldn't be a problem.
 
-        indexData = new IndexData;
+        indexData = OGRE_NEW IndexData;
         indexData->indexBuffer = 
             HardwareBufferManager::getSingleton().createIndexBuffer(
             HardwareIndexBuffer::IT_16BIT,
