@@ -38,7 +38,7 @@ Torus Knot Software Ltd.
 
 #include <d3d9.h>
 #include <d3dx9.h>
-#include <dxerr9.h>
+#include <dxerr.h>
 
 namespace Ogre {
 	class D3D9MultiRenderTarget : public MultiRenderTarget
@@ -47,7 +47,7 @@ namespace Ogre {
 		D3D9MultiRenderTarget(const String &name);
 		~D3D9MultiRenderTarget();
 
-        virtual void update(void);
+        virtual void update(bool swapBuffers = true);
 
 		virtual void getCustomAttribute( const String& name, void *pData );
 
