@@ -364,10 +364,12 @@ namespace Ogre
 			AdjustWindowRect(&rc, GetWindowLong(mHWnd, GWL_STYLE), false);
 			unsigned int winWidth = rc.right - rc.left;
 			unsigned int winHeight = rc.bottom - rc.top;
-			int screenw = GetSystemMetrics(SM_CXSCREEN);
-			int screenh = GetSystemMetrics(SM_CYSCREEN);
-			int left = (screenw - winWidth) / 2;
-			int top = (screenh - winHeight) / 2;
+			unsigned int screenw = GetSystemMetrics(SM_CXSCREEN);
+			unsigned int screenh = GetSystemMetrics(SM_CYSCREEN);
+
+			// deal with overflow when switching down to smaller resolution
+			int left = screenw > winWidth ? ((screenw - winWidth) / 2) : 0;
+			int top = screenh > winHeight ? ((screenh - winHeight) / 2) : 0;
 			SetWindowPos(mHWnd, HWND_NOTOPMOST, left, top, winWidth, winHeight,
 				SWP_DRAWFRAME | SWP_FRAMECHANGED | SWP_NOACTIVATE);
 
@@ -587,7 +589,14 @@ namespace Ogre
 							D3DCREATE_SOFTWARE_VERTEXPROCESSING | extraFlags, &md3dpp, &mpD3DDevice );
 					}
 				}
-				// TODO: make this a bit better e.g. go from pure vertex processing to software
+				
+				if( FAILED( hr ) )
+				{
+					// software device
+					devType = D3DDEVTYPE_REF;
+					hr = pD3D->CreateDevice( adapterToUse, devType, mHWnd,
+						D3DCREATE_SOFTWARE_VERTEXPROCESSING | extraFlags, &md3dpp, &mpD3DDevice );
+				}
 				if( FAILED( hr ) )
 				{
 					destroy();
@@ -880,7 +889,8 @@ namespace Ogre
 					"D3D9RenderWindow::copyContentsToMemory");
 			}
 
-			if (FAILED(hr = mpD3DDevice->GetFrontBufferData(0, pTempSurf)))
+			if (FAILED(hr = mIsSwapChain? mpSwapChain->GetFrontBufferData(pTempSurf) :
+										  mpD3DDevice->GetFrontBufferData(0, pTempSurf)))
 			{
 				SAFE_RELEASE(pTempSurf);
 				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
@@ -945,7 +955,8 @@ namespace Ogre
 		else
 		{
 			SAFE_RELEASE(pSurf);
-			if(FAILED(hr = mpD3DDevice->GetBackBuffer(0, 0, D3DBACKBUFFER_TYPE_MONO, &pSurf)))
+			if(FAILED(hr = mIsSwapChain? mpSwapChain->GetBackBuffer(0, D3DBACKBUFFER_TYPE_MONO, &pSurf) :
+										 mpD3DDevice->GetBackBuffer(0, 0, D3DBACKBUFFER_TYPE_MONO, &pSurf)))
 			{
 				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
 					"Can't get back buffer: " + Root::getSingleton().getErrorDescription(hr),
