@@ -38,7 +38,7 @@ Torus Knot Software Ltd.
 #include "OgreRoot.h"
 
 #include <d3dx9.h>
-#include <dxerr9.h>
+#include <dxerr.h>
 
 namespace Ogre 
 {
@@ -105,7 +105,7 @@ namespace Ogre
 			IDirect3DSurface9 *pSrcSurface = 0;
 			if( FAILED( hr = mpNormTex->GetSurfaceLevel(0, &pSrcSurface) ) )
 			{
-				String msg = DXGetErrorDescription9(hr);
+				String msg = DXGetErrorDescription(hr);
 				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Couldn't blit : " + msg, "D3D9Texture::copyToTexture" );
 			}
 
@@ -114,7 +114,7 @@ namespace Ogre
 			IDirect3DTexture9 *pOthTex = other->getNormTexture();
 			if( FAILED( hr = pOthTex->GetSurfaceLevel(0, &pDstSurface) ) )
 			{
-				String msg = DXGetErrorDescription9(hr);
+				String msg = DXGetErrorDescription(hr);
 				SAFE_RELEASE(pSrcSurface);
 				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Couldn't blit : " + msg, "D3D9Texture::copyToTexture" );
 			}
@@ -122,7 +122,7 @@ namespace Ogre
 			// do the blit, it's called StretchRect in D3D9 :)
 			if( FAILED( hr = mpDev->StretchRect( pSrcSurface, NULL, pDstSurface, &dstRC, D3DTEXF_NONE) ) )
 			{
-				String msg = DXGetErrorDescription9(hr);
+				String msg = DXGetErrorDescription(hr);
 				SAFE_RELEASE(pSrcSurface);
 				SAFE_RELEASE(pDstSurface);
 				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Couldn't blit : " + msg, "D3D9Texture::copyToTexture" );
@@ -143,7 +143,7 @@ namespace Ogre
 				IDirect3DSurface9 *pSrcSurface = 0;
 				if( FAILED( hr = mpCubeTex->GetCubeMapSurface((D3DCUBEMAP_FACES)face, 0, &pSrcSurface) ) )
 				{
-					String msg = DXGetErrorDescription9(hr);
+					String msg = DXGetErrorDescription(hr);
 					OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Couldn't blit : " + msg, "D3D9Texture::copyToTexture" );
 				}
 
@@ -151,7 +151,7 @@ namespace Ogre
 				IDirect3DSurface9 *pDstSurface = 0;
 				if( FAILED( hr = pOthTex->GetCubeMapSurface((D3DCUBEMAP_FACES)face, 0, &pDstSurface) ) )
 				{
-					String msg = DXGetErrorDescription9(hr);
+					String msg = DXGetErrorDescription(hr);
 					SAFE_RELEASE(pSrcSurface);
 					OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Couldn't blit : " + msg, "D3D9Texture::copyToTexture" );
 				}
@@ -159,7 +159,7 @@ namespace Ogre
 				// do the blit, it's called StretchRect in D3D9 :)
 				if( FAILED( hr = mpDev->StretchRect( pSrcSurface, NULL, pDstSurface, &dstRC, D3DTEXF_NONE) ) )
 				{
-					String msg = DXGetErrorDescription9(hr);
+					String msg = DXGetErrorDescription(hr);
 					SAFE_RELEASE(pSrcSurface);
 					SAFE_RELEASE(pDstSurface);
 					OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Couldn't blit : " + msg, "D3D9Texture::copyToTexture" );
@@ -389,7 +389,7 @@ namespace Ogre
             if (FAILED(hr))
 		    {
 				this->freeInternalResources();
-				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't create cube texture: " + String(DXGetErrorDescription9(hr)), 
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't create cube texture: " + String(DXGetErrorDescription(hr)), 
 					"D3D9Texture::_loadCubeTex" );
 		    }
 
@@ -398,7 +398,7 @@ namespace Ogre
             if (FAILED(hr))
 		    {
 				this->freeInternalResources();
-				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture: " + String(DXGetErrorDescription9(hr)), 
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture: " + String(DXGetErrorDescription(hr)), 
 					"D3D9Texture::_loadCubeTex" );
 		    }
 
@@ -484,7 +484,7 @@ namespace Ogre
 			if (FAILED(hr))
 			{
 				OGRE_EXCEPT(Exception::ERR_INTERNAL_ERROR, 
-					"Unable to load volume texture from " + this->getName() + ": " + String(DXGetErrorDescription9(hr)),
+					"Unable to load volume texture from " + this->getName() + ": " + String(DXGetErrorDescription(hr)),
 					"D3D9Texture::_loadVolumeTex");
 			}
 	
@@ -493,7 +493,7 @@ namespace Ogre
 			if (FAILED(hr))
 			{
 				this->freeInternalResources();
-				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture: " + String(DXGetErrorDescription9(hr)), 
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture: " + String(DXGetErrorDescription(hr)), 
 					"D3D9Texture::_loadVolumeTex" );
 			}
 	
@@ -576,7 +576,7 @@ namespace Ogre
 			if (FAILED(hr))
 			{
 				OGRE_EXCEPT(Exception::ERR_INTERNAL_ERROR, 
-					"Unable to load texture from " + this->getName() + " :" + String(DXGetErrorDescription9(hr)),
+					"Unable to load texture from " + this->getName() + " :" + String(DXGetErrorDescription(hr)),
 					"D3D9Texture::_loadNormTex");
 			}
 	
@@ -585,7 +585,7 @@ namespace Ogre
 			if (FAILED(hr))
 			{
 				this->freeInternalResources();
-				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture: " + String(DXGetErrorDescription9(hr)), 
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture: " + String(DXGetErrorDescription(hr)), 
 					"D3D9Texture::_loadNormTex" );
 			}
 	
@@ -620,6 +620,19 @@ namespace Ogre
 
 	}
 	/****************************************************************************************/
+	void D3D9Texture::determinePool()
+	{
+		if (useDefaultPool())
+		{
+			mD3DPool = D3DPOOL_DEFAULT;
+		}
+		else
+		{
+			mD3DPool = D3DPOOL_MANAGED;
+		}
+
+	}
+	/****************************************************************************************/
     void D3D9Texture::createInternalResourcesImpl(void)
 	{
 		// If mSrcWidth and mSrcHeight are zero, the requested extents have probably been set
@@ -629,17 +642,6 @@ namespace Ogre
 			mSrcHeight = mHeight;
 		}
 		
-		// Determine D3D pool to use
-		// Use managed unless we're a render target or user has asked for 
-		// a dynamic texture
-		if (useDefaultPool())
-		{
-			mD3DPool = D3DPOOL_DEFAULT;
-		}
-		else
-		{
-			mD3DPool = D3DPOOL_MANAGED;
-		}
 		// load based on tex.type
 		switch (this->getTextureType())
 		{
@@ -721,6 +723,9 @@ namespace Ogre
 			numMips = 1;
 		}
 
+		// derive the pool to use
+		determinePool();
+
 		// create the texture
 		hr = D3DXCreateTexture(	
 				mpDev,								// device
@@ -735,7 +740,7 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			this->freeInternalResources();
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Error creating texture: " + String(DXGetErrorDescription9(hr)), 
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Error creating texture: " + String(DXGetErrorDescription(hr)), 
 				"D3D9Texture::_createNormTex" );
 		}
 		
@@ -744,7 +749,7 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			this->freeInternalResources();
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture: " + String(DXGetErrorDescription9(hr)), 
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture: " + String(DXGetErrorDescription(hr)), 
 				"D3D9Texture::_createNormTex" );
 		}
 		
@@ -755,7 +760,7 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			this->freeInternalResources();
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get texture description: " + String(DXGetErrorDescription9(hr)), 
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get texture description: " + String(DXGetErrorDescription(hr)), 
 				"D3D9Texture::_createNormTex" );
 		}
 
@@ -771,7 +776,7 @@ namespace Ogre
 			if (FAILED(hr))
 			{
 				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, 
-					"Unable to create AA render target: " + String(DXGetErrorDescription9(hr)), 
+					"Unable to create AA render target: " + String(DXGetErrorDescription(hr)), 
 					"D3D9Texture::_createNormTex");
 			}
 
@@ -785,7 +790,7 @@ namespace Ogre
 			hr = mpTex->SetAutoGenFilterType(_getBestFilterMethod());
 			if(FAILED(hr))
 			{
-				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Could not set best autogen filter type: "  + String(DXGetErrorDescription9(hr)), 
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Could not set best autogen filter type: "  + String(DXGetErrorDescription(hr)), 
 					"D3D9Texture::_createNormTex" );
 			}
 		}
@@ -854,6 +859,9 @@ namespace Ogre
 			numMips = 1;
 		}
 
+		// derive the pool to use
+		determinePool();
+
 		// create the texture
 		hr = D3DXCreateCubeTexture(	
 				mpDev,								// device
@@ -867,7 +875,7 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			this->freeInternalResources();
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Error creating texture: " + String(DXGetErrorDescription9(hr)), 
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Error creating texture: " + String(DXGetErrorDescription(hr)), 
 				"D3D9Texture::_createCubeTex" );
 		}
 
@@ -876,7 +884,7 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			this->freeInternalResources();
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture: " + String(DXGetErrorDescription9(hr)), 
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture: " + String(DXGetErrorDescription(hr)), 
 				"D3D9Texture::_createCubeTex" );
 		}
 		
@@ -887,7 +895,7 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			this->freeInternalResources();
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get texture description: " + String(DXGetErrorDescription9(hr)), 
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get texture description: " + String(DXGetErrorDescription(hr)), 
 				"D3D9Texture::_createCubeTex" );
 		}
 
@@ -903,7 +911,7 @@ namespace Ogre
 			if (FAILED(hr))
 			{
 				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, 
-					"Unable to create AA render target: " + String(DXGetErrorDescription9(hr)), 
+					"Unable to create AA render target: " + String(DXGetErrorDescription(hr)), 
 					"D3D9Texture::_createCubeTex");
 			}
 
@@ -917,7 +925,7 @@ namespace Ogre
 			hr = mpTex->SetAutoGenFilterType(_getBestFilterMethod());
 			if(FAILED(hr))
 			{
-				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Could not set best autogen filter type: " + String(DXGetErrorDescription9(hr)), 
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Could not set best autogen filter type: " + String(DXGetErrorDescription(hr)), 
 					"D3D9Texture::_createCubeTex" );
 			}
 		}
@@ -981,6 +989,9 @@ namespace Ogre
 			numMips = 1;
 		}
 
+		// derive the pool to use
+		determinePool();
+
 		// create the texture
 		hr = D3DXCreateVolumeTexture(	
 				mpDev,								// device
@@ -996,7 +1007,7 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			this->freeInternalResources();
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Error creating texture: " + String(DXGetErrorDescription9(hr)), 
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Error creating texture: " + String(DXGetErrorDescription(hr)), 
 				"D3D9Texture::_createVolumeTex" );
 		}
 
@@ -1005,7 +1016,7 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			this->freeInternalResources();
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture: " + String(DXGetErrorDescription9(hr)), 
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture: " + String(DXGetErrorDescription(hr)), 
 				"D3D9Texture::_createVolumeTex" );
 		}
 		
@@ -1016,7 +1027,7 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			this->freeInternalResources();
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get texture description: " + String(DXGetErrorDescription9(hr)), 
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get texture description: " + String(DXGetErrorDescription(hr)), 
 				"D3D9Texture::_createVolumeTex" );
 		}
 		this->_setFinalAttributes(desc.Width, desc.Height, desc.Depth, D3D9Mappings::_getPF(desc.Format));
@@ -1027,7 +1038,7 @@ namespace Ogre
 			hr = mpTex->SetAutoGenFilterType(_getBestFilterMethod());
 			if(FAILED(hr))
 			{
-				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Could not set best autogen filter type: " + String(DXGetErrorDescription9(hr)), 
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Could not set best autogen filter type: " + String(DXGetErrorDescription(hr)), 
 					"D3D9Texture::_createCubeTex" );
 			}
 		}
@@ -1041,7 +1052,7 @@ namespace Ogre
 		// get device caps
 		hr = mpDev->GetDeviceCaps(&mDevCaps);
 		if (FAILED(hr))
-			OGRE_EXCEPT( Exception::ERR_INTERNAL_ERROR, "Can't get device description: " + String(DXGetErrorDescription9(hr)), 
+			OGRE_EXCEPT( Exception::ERR_INTERNAL_ERROR, "Can't get device description: " + String(DXGetErrorDescription(hr)), 
 				"D3D9Texture::_setDevice" );
 
 		// get D3D pointer
@@ -1049,13 +1060,13 @@ namespace Ogre
 		// decrement reference count
 		mpD3D->Release();
 		if (FAILED(hr))
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Failed to get D3D9 pointer: " + String(DXGetErrorDescription9(hr)), 
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Failed to get D3D9 pointer: " + String(DXGetErrorDescription(hr)), 
 				"D3D9Texture::_setDevice" );
 
 		// get our device creation parameters
 		hr = mpDev->GetCreationParameters(&mDevCreParams);
 		if (FAILED(hr))
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Failed to get D3D9 device creation parameters: " + String(DXGetErrorDescription9(hr)), 
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Failed to get D3D9 device creation parameters: " + String(DXGetErrorDescription(hr)), 
 				"D3D9Texture::_setDevice" );
 
 		// get our back buffer pixel format
@@ -1065,13 +1076,13 @@ namespace Ogre
 		// decrement reference count
 		pSrf->Release();
 		if (FAILED(hr))
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Failed to get D3D9 device pixel format: " + String(DXGetErrorDescription9(hr)), 
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Failed to get D3D9 device pixel format: " + String(DXGetErrorDescription(hr)), 
 				"D3D9Texture::_setDevice" );
 
 		hr = pSrf->GetDesc(&srfDesc);
 		if (FAILED(hr))
 		{
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Failed to get D3D9 device pixel format: " + String(DXGetErrorDescription9(hr)), 
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Failed to get D3D9 device pixel format: " + String(DXGetErrorDescription(hr)), 
 				"D3D9Texture::_setDevice" );
 		}
 
@@ -1425,7 +1436,11 @@ namespace Ogre
 	/****************************************************************************************/
 	bool D3D9Texture::useDefaultPool()
 	{
-		return (mUsage & TU_RENDERTARGET) || (mUsage & TU_DYNAMIC);
+		// Determine D3D pool to use
+		// Use managed unless we're a render target or user has asked for 
+		// a dynamic texture, and device supports D3DUSAGE_DYNAMIC (because default pool
+		// resources without the dynamic flag are not lockable)
+		return (mUsage & TU_RENDERTARGET) || ((mUsage & TU_DYNAMIC) && mDynamicTextures);
 	}
 	/****************************************************************************************/
 	bool D3D9Texture::releaseIfDefaultPool(void)
@@ -1485,14 +1500,14 @@ namespace Ogre
 
 
 	/****************************************************************************************/
-    void D3D9RenderTexture::update(void)
+    void D3D9RenderTexture::update(bool swapBuffers)
     {
         D3D9RenderSystem* rs = static_cast<D3D9RenderSystem*>(
             Root::getSingleton().getRenderSystem());
         if (rs->isDeviceLost())
             return;
 
-        RenderTexture::update();
+        RenderTexture::update(swapBuffers);
     }
 	//---------------------------------------------------------------------
 	void D3D9RenderTexture::getCustomAttribute( const String& name, void *pData )
@@ -1541,7 +1556,7 @@ namespace Ogre
 			if (FAILED(hr))
 			{
 				OGRE_EXCEPT(Exception::ERR_INTERNAL_ERROR, 
-					"Unable to copy AA buffer to final buffer: " + String(DXGetErrorDescription9(hr)), 
+					"Unable to copy AA buffer to final buffer: " + String(DXGetErrorDescription(hr)), 
 					"D3D9RenderTexture::swapBuffers");
 			}
 			
