@@ -35,8 +35,8 @@ Torus Knot Software Ltd.
 
 namespace Ogre 
 {
-	GLXContext::GLXContext(GLXGLSupport* glsupport, ::GLXFBConfig fbconfig, ::GLXDrawable drawable) :
-		mGLSupport(glsupport), mDrawable(drawable), mContext(0), mFBConfig(fbconfig)
+	GLXContext::GLXContext(GLXGLSupport* glsupport, ::GLXFBConfig fbconfig, ::GLXDrawable drawable, ::GLXContext context) :
+		mGLSupport(glsupport), mDrawable(drawable), mContext(0), mFBConfig(fbconfig), mExternalContext(false)
 	{
 		GLRenderSystem *renderSystem = static_cast<GLRenderSystem*>(Root::getSingleton().getRenderSystem());
 		GLXContext* mainContext = static_cast<GLXContext*>(renderSystem->_getMainContext());
@@ -47,8 +47,16 @@ namespace Ogre
 			shareContext = mainContext->mContext;
 		}
 		
-		mContext = mGLSupport->createNewContext(mFBConfig, GLX_RGBA_TYPE, shareContext, GL_TRUE);
-		
+		if (context)
+		{
+			mContext = context;
+			mExternalContext = true;
+		}
+		else
+		{
+			mContext = mGLSupport->createNewContext(mFBConfig, GLX_RGBA_TYPE, shareContext, GL_TRUE);
+		}
+
 		if (! mContext)
 		{
 			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Unable to create a suitable GLXContext", "GLXContext::GLXContext");
@@ -59,7 +67,8 @@ namespace Ogre
 	{
 		GLRenderSystem *rs = static_cast<GLRenderSystem*>(Root::getSingleton().getRenderSystem());
 		
-		glXDestroyContext(mGLSupport->getGLDisplay(), mContext);
+		if (!mExternalContext)
+			glXDestroyContext(mGLSupport->getGLDisplay(), mContext);
 		
 		rs->_unregisterContext(this);
 	}
