@@ -404,7 +404,7 @@ namespace Ogre
 			WindowEventUtilities::_addRenderWindow(this);
 		}
 		
-		mContext = new GLXContext(mGLSupport, fbConfig, glxDrawable);
+    mContext = new GLXContext(mGLSupport, fbConfig, glxDrawable, glxContext);
 		
 		::GLXDrawable oldDrawable = glXGetCurrentDrawable();
 		::GLXContext  oldContext  = glXGetCurrentContext();
@@ -530,7 +530,7 @@ namespace Ogre
 		
 		if(width != 0 && height != 0)
 		{
-			if (mIsTopLevel)
+			if (!mIsExternal)
 			{
 				XResizeWindow(mGLSupport->getXDisplay(), mWindow, width, height);
 			}
