@@ -183,7 +183,7 @@ void OSXGLSupport::addConfig( void )
 	optRTTMode.possibleValues.push_back( "FBO" );
 	optRTTMode.possibleValues.push_back( "PBuffer" );
 	optRTTMode.possibleValues.push_back( "Copy" );
-	optRTTMode.currentValue = "PBuffer";
+	optRTTMode.currentValue = "FBO";
 	optRTTMode.immutable = false;
 
 
@@ -229,7 +229,7 @@ RenderWindow* OSXGLSupport::createWindow( bool autoCreateWindow, GLRenderSystem*
 			winOptions[ "FSAA" ] = opt->second.currentValue;
         }
 
-		return renderSystem->createRenderWindow( windowTitle, w, h, fullscreen, &winOptions );
+		return renderSystem->_createRenderWindow( windowTitle, w, h, fullscreen, &winOptions );
 	}
 	else
 	{
