@@ -77,13 +77,13 @@ void GLHardwarePixelBuffer::freeBuffer()
 PixelBox GLHardwarePixelBuffer::lockImpl(const Image::Box lockBox,  LockOptions options)
 {
 	allocateBuffer();
-	if(options != HardwareBuffer::HBL_DISCARD
-		&& (mUsage & HardwareBuffer::HBU_WRITE_ONLY) == 0) 
+	if(options != HardwareBuffer::HBL_DISCARD) 
 	{
 		// Download the old contents of the texture
 		download(mBuffer);
 	}
 	mCurrentLockOptions = options;
+	mLockedBox = lockBox;
 	return mBuffer.getSubVolume(lockBox);
 }
 //-----------------------------------------------------------------------------  
@@ -92,7 +92,7 @@ void GLHardwarePixelBuffer::unlockImpl(void)
 	if (mCurrentLockOptions != HardwareBuffer::HBL_READ_ONLY)
 	{
 		// From buffer to card, only upload if was locked for writing
-		upload(mCurrentLock, Box(0,0,0,mWidth,mHeight,mDepth));
+		upload(mCurrentLock, mLockedBox);
 	}
 	
 	freeBuffer();
@@ -127,6 +127,7 @@ void GLHardwarePixelBuffer::blitFromMemory(const PixelBox &src, const Image::Box
 	}
 	else
 	{
+		allocateBuffer();
 		// No scaling or conversion needed
 		scaled = src;
 	}
@@ -456,7 +457,8 @@ void GLTextureBuffer::upload(const PixelBox &data, const Image::Box &dest)
 	}
 	// Restore defaults
 	glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
-	glPixelStorei(GL_UNPACK_IMAGE_HEIGHT, 0);
+	if (GLEW_VERSION_1_2)
+		glPixelStorei(GL_UNPACK_IMAGE_HEIGHT, 0);
 	glPixelStorei(GL_UNPACK_SKIP_PIXELS, 0);
 	glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
 }
@@ -579,7 +581,15 @@ void GLTextureBuffer::blitFromTexture(GLTextureBuffer *src, const Image::Box &sr
         GL_FOG_BIT | GL_LIGHTING_BIT | GL_POLYGON_BIT | GL_SCISSOR_BIT | GL_STENCIL_BUFFER_BIT |
         GL_TEXTURE_BIT | GL_VIEWPORT_BIT);
 
-    /// Disable alpha, depth and scissor testing, disable blending, 
+	// Important to disable all other texture units
+	RenderSystem* rsys = Root::getSingleton().getRenderSystem();
+	rsys->_disableTextureUnitsFrom(1);
+	if (GLEW_VERSION_1_2)
+	{
+		glActiveTextureARB(GL_TEXTURE0);
+	}
+
+	/// Disable alpha, depth and scissor testing, disable blending, 
     /// disable culling, disble lighting, disable fog and reset foreground
     /// colour.
     glDisable(GL_ALPHA_TEST);
