@@ -34,6 +34,8 @@ Torus Knot Software Ltd.
 #include <vector>
 
 #ifdef _WIN32
+#define WIN32_LEAN_AND_MEAN
+#define NOMINMAX // required to stop windows.h messing up std::min
 #include <windows.h>
 #endif
 
