@@ -126,11 +126,13 @@ Torus Knot Software Ltd.
 
 */
 
-void yyerror(char* s);
+void yyerror(const char* s);
 int yylex ( void );
 
 #ifdef _WIN32
-# include <windows.h>
+#  define WIN32_LEAN_AND_MEAN
+#  define NOMINMAX // required to stop windows.h messing up std::min
+#  include <windows.h>
 #endif
 
 #include <stdio.h>
@@ -1547,7 +1549,7 @@ yyreturn:
 #line 309 "ps1.0_grammar.y"
 
 
-void yyerror(char* s)
+void yyerror(const char* s)
 {
 	errors.set("parser: syntax error", line_number);
 }
