@@ -165,11 +165,13 @@ Torus Knot Software Ltd.
 /* Copy the first part of user declarations.  */
 #line 3 "rc1.0_grammar.y"
 
-void yyerror(char* s);
+void yyerror(const char* s);
 int yylex ( void );
 
 #ifdef _WIN32
-# include <windows.h>
+#  define WIN32_LEAN_AND_MEAN
+#  define NOMINMAX // required to stop windows.h messing up std::min
+#  include <windows.h>
 #endif
 
 #include <stdio.h>
@@ -2314,7 +2316,7 @@ yyreturn:
 
 #line 679 "rc1.0_grammar.y"
 
-void yyerror(char* s)
+void yyerror(const char* s)
 {
      errors.set("unrecognized token");
 }
