@@ -193,11 +193,13 @@ Torus Knot Software Ltd.
 /* Copy the first part of user declarations.  */
 #line 3 "ts1.0_grammar.y"
 
-void yyerror(char* s);
+void yyerror(const char* s);
 int yylex ( void );
 
 #ifdef _WIN32
-# include <windows.h>
+#  define WIN32_LEAN_AND_MEAN
+#  define NOMINMAX // required to stop windows.h messing up std::min
+#  include <windows.h>
 #endif
 #include <stdio.h>
 #include <stdlib.h>
@@ -1831,7 +1833,7 @@ yyreturn:
 
 #line 300 "ts1.0_grammar.y"
 
-void yyerror(char* s)
+void yyerror(const char* s)
 {
      errors.set("unrecognized token");
 }
