@@ -1,5 +1,7 @@
 #ifdef _WIN32
-# include <windows.h>
+#  define WIN32_LEAN_AND_MEAN
+#  define NOMINMAX // required to stop windows.h messing up std::min
+#  include <windows.h>
 #else
 #include <stdarg.h>
 #define strnicmp strncasecmp
