@@ -118,6 +118,11 @@ namespace OgreMayaExporter
 			std::cout << "Error retrieving blend shape deformer linked to current mesh\n";
 			std::cout.flush();
 		}
+		// Set blend shape defeormer envelope to 0, to get base mesh
+		if (pBlendShape)
+		{
+			pBlendShape->setEnvelope(0.0f);
+		}
 		// Get connected shaders
 		stat = getShaders(meshDag);
 		if (stat != MS::kSuccess)
@@ -166,6 +171,11 @@ namespace OgreMayaExporter
 			std::cout << "Error creating submeshes for current mesh\n";
 			std::cout.flush();
 		}
+		// Restore blendshape envelope
+		if (pBlendShape)
+		{
+			pBlendShape->restoreEnvelope();
+		}
 		// Free up memory
 		newvertices.clear();
 		newweights.clear();
@@ -1291,7 +1301,8 @@ namespace OgreMayaExporter
 				canBuild = false;
 			}
 			if (canBuild)
-				pMesh->buildTangentVectors(targetSemantic, srcTex, destTex);
+				pMesh->buildTangentVectors(targetSemantic, srcTex, destTex, 
+					params.tangentsSplitMirrored, params.tangentsSplitRotated, params.tangentsUseParity);
 		}
 		// Export the binary mesh
 		Ogre::MeshSerializer serializer;
