@@ -108,6 +108,7 @@ namespace Ogre {
 	DeformerMap& XsiMeshExporter::buildMeshForExport(
 		bool mergeSubMeshes, bool exportChildren, 
 		bool edgeLists, bool tangents, VertexElementSemantic tangentSemantic, 
+		bool tangentsSplitMirrored, bool tangentsSplitRotated, bool tangentsUseParity,
 		bool vertexAnimation, AnimationList& animList, Real fps, const String& materialPrefix, 
 		LodData* lod, const String& skeletonName)
     {
@@ -165,7 +166,8 @@ namespace Ogre {
             unsigned short src, dest;
             if (!mMesh->suggestTangentVectorBuildParams(tangentSemantic, src, dest))
             {
-                mMesh->buildTangentVectors(tangentSemantic, src, dest);
+                mMesh->buildTangentVectors(tangentSemantic, src, dest, tangentsSplitMirrored, 
+					tangentsSplitRotated, tangentsUseParity);
             }
             else
             {
@@ -577,11 +579,11 @@ namespace Ogre {
 				samplerUVs[samplerIndex].y = 1.0f - curUVW[1];//v (invert)
 				samplerUVs[samplerIndex].z = curUVW[2];//w
 
-				if (!hasU && curUVW[0] != 0)
+				if (!hasU && !Ogre::Math::RealEqual(curUVW[0], 0))
 					hasU = true;
-				if (!hasV && curUVW[1] != 0)
+				if (!hasV && !Ogre::Math::RealEqual(curUVW[1], 0))
 					hasV = true;
-				if (!hasW && curUVW[2] != 0)
+				if (!hasW && !Ogre::Math::RealEqual(curUVW[2], 0))
 					hasW = true;
 
 			}
@@ -1028,7 +1030,10 @@ namespace Ogre {
 		// Process all mixers
 		Model root = mXsiApp.GetActiveSceneRoot();
 		if (root.HasMixer())
-			buildShapeClipList(root.GetMixer(), listToPopulate);
+		{
+			XSI::Mixer mixer = root.GetMixer();
+			buildShapeClipList(mixer, listToPopulate);
+		}
 
 		// Get all child models (recursive)
 		CRefArray models = root.GetModels();
@@ -1036,7 +1041,10 @@ namespace Ogre {
 		{
 			Model model(models[m]);
 			if (model.HasMixer())
-				buildShapeClipList(model.GetMixer(), listToPopulate);
+			{
+				XSI::Mixer mixer = root.GetMixer();
+				buildShapeClipList(mixer, listToPopulate);
+			}
 		}
 
 	}
